/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.core.nodeiterators;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeGenerator;
import com.unitesk.requality.core.AttributeGenerator.ParameterAvailability;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.INodeChangeListener;
import com.unitesk.requality.core.IStorageDeltaListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeLogic;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.instancer.TreeInstancer;
import com.unitesk.requality.nodetypes.VirtualNode;

public class ReuseRequalityChildIterator extends RequalityChildIterator{
	//private String idPredicate = "$i";
	protected Map<UUID,Map<String,INodeChangeListener>> idAttributeListeners = new HashMap<UUID,Map<String,INodeChangeListener>>();
	private List<String>listAttrs = new ArrayList<String>();
	@Override
	public INodeChangeListener getMasterListener(){
		return listener;
	}
	private INodeChangeListener listener = new INodeChangeListener() {
		@Override
		public void moved(TreeNode source, TreeNode oldParent, TreeNode newParent) {
		}
		
		@Override
		public void deleted(TreeNode source) {
		}
		
		@Override
		public void attributeChange(TreeNode source, String what) {
			getMaster(source.getTreeDB()).removeChangeListener(listener);
			if(listAttrs.contains(what)){
				if(source.getAttribute(what)==null || !source.getAttribute(what).getType().equals(AttributeType.LIST)){
					while(listAttrs.contains(what))
						listAttrs.remove(what);
					((VirtualNode)source).saveChildIterator();
					((VirtualNode)source).saveAttributes();
					
				}
				getInstancerMaster(source.getTreeDB()).removeClones();
			}
			getMaster(source.getTreeDB()).addChangeListener(listener);
		}
	};
	
	public ReuseRequalityChildIterator(UUID guid, UUID mast, String s){
		super(guid,mast,RequalityIteratorType.CHILD,s);
		
		chitype = RequalityChildIteratorType.REUSE_CARTESIAN;
		String[]del = s.split("\\|");
		
		int cnt = Integer.parseInt(safeGet(del, 1, "0"));
		for(int i = 0;i<cnt;i++){
			String var = safeGet(del, i+2);
			if(var.startsWith("\""))
				var = var.substring(1,var.length()-1);
			listAttrs.add(var);
		}
    }
	
	private String safeGet(String[]del, int s){
		return safeGet(del, s, "");
	}
	//prefer to use it. for some cases del will be empty
	private String safeGet(String[]del, int s, String def){
		if(s >= del.length || del[s].equals(""))
			return def;
		return del[s];
	}
	
    
	@Override
    public String toString() {
		//TODO change
        return "type: "+type+" target: ";
    }

    @SuppressWarnings("unchecked")
    @Override
    public boolean equals(Object obj) {
        if(super.equals(obj)) return true;
        if (!(obj instanceof ReuseRequalityChildIterator)) {
            return false;
        }
        ReuseRequalityChildIterator it = (ReuseRequalityChildIterator)obj;
        return super.equals(obj);
    }
    
    public String toAttributeString() {
		String s = super.toAttributeString()+"|"+getChildIteratorType().ordinal()+"|"+listAttrs.size();
		for(int i = 0;i<listAttrs.size();i++)
			s+="|\""+listAttrs.get(i)+"\"";
		return s;
	}
//    
//    public String getNodeId(int i, TreeNode node){
//    	return idGenerator.getValue(i, node).toString();
////    	updateDependencies(node);
//  //  	return node.evaluatePredicateString(node.getUUId(), 
//    //			RequalityAttributeGenerator.replaceRegexpVar(idPredicate, "$i", i+""));
//    }
//    
	public int getCount(TreeDB db) {
		TreeNode master = db.getNode(getMasterUUId());
		Collection<Attribute> attrs = master.getAttributes().values();
		switch (getChildIteratorType()) {
			case REUSE_CARTESIAN:
			{
				int cnt = 1;
				for(String val:listAttrs)
				{
					Attribute a = master.getAttribute(val);
					if(a!=null){
						AttributeGenerator rag = a.getGenerator();
						if(rag!=null)
							cnt*=rag.getItemsCount();
						else{
							Attribute b=master.getAttribute(val);
							if(b!=null&&b.getType().equals(AttributeType.LIST)){
								cnt*=((List<String>)b.getValue()).size();	
							}
						}
					}
				}
				return cnt-ignoredIds.size();
			}

		default:
			break;
		}
		return 0;
	}
	
	public List<String> getAttributeVars(){
		return listAttrs;
	}
	
	public void setAttributeVar(TreeDB db, int num, String value) {
		if(num < 0 || num >= listAttrs.size())
			return;
		if(!listAttrs.get(num).equals(value)&&db.getNode(getMasterUUId()).getAttribute(value)!=null){
			getInstancerMaster(db).removeClones();
			listAttrs.set(num, value);
			getInstancerMaster(db).saveChildIterator();
		}
	}
	
	public List<String> getAttributeNames(TreeDB db){
		List<String> nm = new ArrayList<String>();
		for(Attribute a: getMaster(db).getAttributes().values())
			if(a.getType().equals(AttributeType.LIST) && !a.getKey().startsWith("_"))
				nm.add(a.getKey());
		return nm;
	}
	
	public void addAttributeVar(TreeDB db)  {
		getInstancerMaster(db).removeClones();
		String key = "";
		//Set<String> keys = getInstancerMaster().getAvailableGenerators(false).keySet();
		//if(!keys.isEmpty())
			//key = keys.iterator().next();
		listAttrs.add(getAttributeNames(db).get(0));
		getInstancerMaster(db).saveChildIterator();
	}
	
	public void removeAttributeVar(TreeDB db, int num) {
		if(num < 0 || num >= listAttrs.size())
			return;
		getInstancerMaster(db).removeClones();
		listAttrs.remove(num);
		getInstancerMaster(db).saveChildIterator();
		getInstancerMaster(db).saveAttributes();
	}

	public String getAttributeVar(int i) {
		if(i < 0 || i >= listAttrs.size())
			return "";
		return listAttrs.get(i);
	}
	private List<Integer> ignoredIds = new ArrayList<Integer>();
	public void updateChildren(TreeDB db) {
		if(getInstancerMaster(db).getTarget() == null)
			return;;
		TreeInstancer ti = getInstancerMaster(db).getVirtualStorage();
		getInstancerMaster(db).getVirtualTreeDB().startChangeRow();
		int count = getItemsCount(db);
		int ii = 0;

		for(UUID uid : getInstancerMaster(db).getClones(true)){
			try{
				TreeNode node = getInstancerMaster(db).getVirtualTreeDB().getNode(uid);
				//AttributeGenerator ag = node.getGenerator(TreeNode.ATTR_NAME);
				//if(ag!=null)
					//node.removeIterator(ag);
				Attribute a = node.getAttribute("__i");
				if(a!=null){
					int i = Integer.parseInt(a.getValue().toString());
					if(i>=count){
						//node.delete();
						ti.remove(node.getUUId(),true);
					}
				}
			}
			catch(Exception e){
				//cannot remove
			}
			//ti.saveChanges();
		}
		int[]cnt = new int[listAttrs.size()];
		int[]max = new int[listAttrs.size()];
		for(int i = 0;i<max.length;i++)
			if(!listAttrs.get(i).equals("")){

				Object rawValue =  getMaster(db).getAttributeValue(listAttrs.get(i));
				if(rawValue instanceof List)
					max[i] = ((List)rawValue).size();
				else
					if(rawValue!=null)
						max[i] = 1;
					else{
						Attribute a = getMaster(db).getAttribute(listAttrs.get(i));
						if(a!=null){
							AttributeGenerator gen = a.getGenerator();
							if(gen!=null)
								max[i] = gen.getItemsCount();
						}
					}
			}
		ignoredIds = new ArrayList<Integer>();
		for(int i=0; i<count; i++){
			long k = new Date().getTime();
			String newId = "clone_"+Integer.toString(i);
			UUID cloneuid = ti.cloneUUID(getInstancerMaster(db).getTarget().getUUId(), getInstancerMaster(db).getUUId(),newId);
			TreeNode node = null;
			try{
				node = getInstancerMaster(db).getVirtualTreeDB().getNode(cloneuid);
			}catch(Exception e){ 
				e.printStackTrace();
			}
			node.putAttribute(new Attribute(getMaster(db).getUUId(), AttributeType.STRING, "__i", Integer.toString(i)));
			node.putAttribute(new Attribute(getMaster(db).getUUId(), AttributeType.STRING, "__I", Integer.toString(i+1)));
			Map<String, Integer> names = new HashMap<String,Integer>();
			for(int j=0;j<max.length;j++)
				if(!listAttrs.get(j).equals("")){
					String key = listAttrs.get(j);
					List<String> vals = new ArrayList<String>();
					Object rawValue = getMaster(db).getAttributeValue(key);
					if(rawValue instanceof List)
						vals.addAll((List<String>)rawValue);
					else
						if(rawValue!=null)
							vals.add(rawValue.toString());
					if(vals!=null && !vals.isEmpty())
					{
						long l = new Date().getTime();
						AttributeGenerator ag = null;
						String attrName = key;
						if(!names.containsKey(key))
							names.put(key, 0);
						names.put(key, names.get(key)+1);
						if(names.get(key)>1)
							attrName += "_"+names.get(key);
						Attribute a = new Attribute(node,AttributeType.GENERATED,attrName,"");
						Map<String,Object> settings = new HashMap<String, Object>();
						settings.put(AttributeGenerator.SETT_ITERTYPE, AttributeValueIteratorType.fromId(AttributeValueIteratorType.FORMULA_ID));
						settings.put(AttributeGenerator.SETT_AVAILABILITY, ParameterAvailability.SUBTREE);
						settings.put(AttributeGenerator.SETT_FORMULA,"\""+ vals.get(cnt[j]).toString()+"\"");
						
						ag = new AttributeGenerator(a, UUID.randomUUID(), settings);
						a.setGenerator(ag);
						node.putAttribute(a);
						//ag = node.addAttributeIterator(UUID.randomUUID(),node,ParameterAvailability.SUBTREE.ordinal()+"|"+key+"|"+AttributeValueIteratorType.FORMULA_ID+"|\""+vals.get(cnt[j]).toString()+"\"");
						//node.saveAttributeIterators();
					}
				}
			IStorageDeltaListener del = ti.getDeltaTarget();
			if(!newId.equals(node.getId())){
				node.setId(newId);
			}
			if(cnt.length > 0)
				cnt[0]++;
			int j = 0;
			while(j<cnt.length && cnt[j]>=max[j]){
				cnt[j] = 0;
				if(j+1<cnt.length)
					cnt[j+1]++;
				j++;
			}
			node.saveAttributes();
		}
		//ti.saveChanges();
		getInstancerMaster(db).getVirtualTreeDB().stopChangeRow();
	}

	public List<Integer> getIgnored() {
		return ignoredIds;
	}

	@Override
	protected String checkPossibleTarget(TreeNode newtarget) {
		TreeNode par = getInstancerMaster(newtarget.getTreeDB()).getParent();
		TreeLogic ti = newtarget.getTreeDB().getTreeLogic();
		if(!ti.canBeChildOf(newtarget.getType(), par))
			return "'"+newtarget.getUserFriendlyName()+"' cannot be child of '"+par.getUserFriendlyName()+"'";
		return null;
	}
	@Override 
	public void updateDependencies(TreeDB db){
		TreeNode master = getMaster(db);
		for(String s:listAttrs)
			{
				Attribute at = master.getAttribute(s);
				if(at!=null){
					TreeNode origin = master.getTreeDB().getNode(at.getOrigin());
					Set<String> deps = dependencies.get(origin);
					if(deps == null)
						deps = new HashSet<String>();
					dependencies.put(origin, deps);
				}
			}
	}
}
