/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.core;

import java.util.UUID;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;

public class MoveNodeJob implements IWorkspaceRunnable {
    private IFile toFile;
    private UUID curNodeUUId;
    private UUID oldParentUUID;
    private UUID newParentUUId;
    private ResourceStorage rs;

    public MoveNodeJob(ResourceStorage rs, IFile toFile, UUID curNodeUUId,
            UUID oldParentUUID, UUID newParentUUId) {
        this.rs = rs;
        this.toFile = toFile;
        this.curNodeUUId = curNodeUUId;
        this.oldParentUUID = oldParentUUID;
        this.newParentUUId = newParentUUId;
    }

    private String getNodeId(IFile file) {
        if (!file.getFileExtension().equals("json")) {
            throw new IllegalArgumentException("File must be .json");
        }
        return file.getName().substring(0, file.getName().length() - 5);
    }

    @Override
    public void run(IProgressMonitor monitor) throws CoreException {
        IFile fromFile = rs.getResource(curNodeUUId);

        IFolder newFolder = (IFolder) toFile.getParent();
        if (!newFolder.exists()) {
            newFolder.create(false, true, null);
        }

        IFolder nodeChildrenFolder = fromFile.getParent().getFolder(
                new Path(getNodeId(toFile)));

        IFolder oldNodeFolder = (IFolder) fromFile.getParent();
        fromFile.move(toFile.getFullPath(), false, null);
        IPath oldChildrenLocation = nodeChildrenFolder.getLocation();
        IPath newChildrenLocation = null;
        if (nodeChildrenFolder.exists()) {
            if (nodeChildrenFolder.members().length > 0) {
                IFolder folder= newFolder.getFolder(
                        Path.fromOSString(getNodeId(fromFile)));
                IPath path = folder.getFullPath();
                newChildrenLocation = folder.getLocation();
                nodeChildrenFolder.move(path, false, null);
            } else {
                nodeChildrenFolder.delete(false, null);
            }
        }
        IFolder resFolder = fromFile.getParent().getFolder(
                new Path(getNodeId(toFile) + ResourceStorage.RESOURCE_POSTFIX));

        if (resFolder.exists()) {
            if (resFolder.members().length > 0) {
                IPath path = newFolder.getFolder(
                        Path.fromOSString(getNodeId(fromFile)
                                + ResourceStorage.RESOURCE_POSTFIX))
                        .getFullPath();
                resFolder.move(path, false, null);
            } else {
                resFolder.delete(false, null);
            }
        }

        if (oldNodeFolder.members().length == 0) {
            oldNodeFolder.delete(false, null);
        }

        if(newChildrenLocation!=null) {
            rs.getResourceMap().updatePath(oldChildrenLocation, newChildrenLocation);
        }
        rs.getResourceMap().cacheReplace(fromFile, toFile, curNodeUUId);
        rs.getDeltaTarget().moved(curNodeUUId, oldParentUUID, newParentUUId, true);
    }
}
