package com.unitesk.requality.eclipse.ui;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.model.WorkbenchLabelProvider;
import org.eclipse.ui.navigator.CommonNavigator;
import org.eclipse.ui.navigator.CommonViewer;
import org.eclipse.ui.progress.UIJob;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.core.IUIStateChangeListener;
import com.unitesk.requality.eclipse.ui.cnf.RequalitySorter;
import com.unitesk.requality.eclipse.views.RequalityContentLinksProvider;
import com.unitesk.requality.eclipse.views.RequalityContentLinksProvider.TestNodeLinks;
public class RequalityLCNF extends CommonNavigator implements ITabbedPropertySheetPageContributor {
	public static final String ID = "com.unitesk.requality.views.RequalityLinksExplorer";
	private static byte refdir = 3;
	private boolean updating = false;
	private static List<RequalityLCNF> activeNaivgators = new ArrayList<RequalityLCNF>();
	public RequalityLCNF(){
		super();
		activeNaivgators.add(this);
	}
    @Override
    protected CommonViewer createCommonViewerObject(Composite aParent) {
        final CommonViewer cv = new CommonViewer(getViewSite().getId(), aParent,
                SWT.H_SCROLL | SWT.V_SCROLL | SWT.MULTI);
        cv.setContentProvider(new RequalityContentLinksProvider());
        final ILabelProvider base = WorkbenchLabelProvider.getDecoratingWorkbenchLabelProvider();
        cv.addDoubleClickListener(new IDoubleClickListener() {
			
			@Override
			public void doubleClick(DoubleClickEvent event) {
				if(updating)
					return;
				updating = true;
				StructuredSelection s = (StructuredSelection) event.getSelection();
				if(s!=null && !s.isEmpty()){
					Object o = s.getFirstElement();
					TreeNode node = null;
					if(o instanceof TreeNode){
						node = (TreeNode)o;
					}
					else if(o instanceof TestNodeLinks)
						node = ((TestNodeLinks)o).getTarget();
					if(node!=null){
						HashMap<String, List<TreeNode>> nodes = new HashMap<String, List<TreeNode>>();
						nodes.put(node.getType(), new ArrayList<TreeNode>());
						nodes.get(node.getType()).add(node);
						Activator.getDefault().setActiveNodes(nodes);
					}
				}
				updating = false;
			}
		});
        cv.setLabelProvider(new ILabelProvider() {
			@Override
			public void removeListener(ILabelProviderListener listener) {
				base.removeListener(listener);
			}
			
			@Override
			public boolean isLabelProperty(Object element, String property) {
				return true;
			}
			
			@Override
			public void dispose() {
				base.dispose();
			}
			
			@Override
			public void addListener(ILabelProviderListener listener) {
				base.addListener(listener);
			}
			
			@Override
			public String getText(Object element) {
				String baseText = base.getText(element);
				if(!baseText.equals(""))
					return baseText;
				return element.toString();
			}
			
			@Override
			public Image getImage(Object element) {
				if(element instanceof TestNodeLinks){
					String imgup = "arrowup",imgdown="arrowdown";
					TestNodeLinks link = (TestNodeLinks) element;
					return Activator.getDefault().getImageRegistry().getDescriptor(link.isUp()?imgup:imgdown).createImage();
					
				}
				return base.getImage(element);
			}
		});
        Activator.getDefault().addChangeListener(new IUIStateChangeListener() {
			
			@Override
			public void newActiveNodes(final List<TreeNode> nodes) {
				 new UIJob("update requality link cnf") {
					@Override
					public IStatus runInUIThread(IProgressMonitor monitor) {
						if(updating)
							return Status.CANCEL_STATUS;
						if(cv == null || cv.getControl().isDisposed()){
							return Status.CANCEL_STATUS;
						}
						updating = true;
						if(nodes.size()>0)
							cv.setInput(nodes.get(0));
						else
							cv.setInput(null);
						updating = false;
						return Status.OK_STATUS;
					}
				}.schedule(); 
				
			}
		});
        return cv;
    }
    

    @Override
    public String getContributorId() {
        return "com.unitesk.requality.views.RequalityPropertyContributor";
    }
    @Override
    public void createPartControl(Composite aParent) {
        super.createPartControl(aParent);
        getCommonViewer().setSorter(new RequalitySorter());
    }
    @Override
    public void dispose() {
    	super.dispose();
    	activeNaivgators.remove(this);
    };
    @Override
    public Object getAdapter(@SuppressWarnings("rawtypes") Class adapter) {
        if (adapter == IPropertySheetPage.class) {
            return new TabbedPropertySheetPage(this);
        }
        return super.getAdapter(adapter);
    }
    @Override
    protected Object getInitialInput() {
    	if(super.getInitialInput()==null) {
    		return Activator.getDefault().getActiveNodes().values().iterator().next();
    	}
    	return super.getInitialInput();
    }


	public static void toggleReferenceDirection() {
		refdir++;
		if(refdir == 4)
			refdir = 1;
		for(RequalityLCNF cnf:activeNaivgators){
			cnf.getCommonViewer().refresh(true);
		}
	}


	public static int getReferenceDirection() {
		return refdir;
	}

    private static Map<TreeDB,Set<String>> filter = new HashMap<TreeDB, Set<String>>();
	public static Set<String> getFilters(TreeDB treeDB) {
		return filter.get(treeDB);
	}

	public static void setFilter(TreeDB treeDB, HashSet<String> hashSet) {
		filter.put(treeDB, hashSet);
		for(RequalityLCNF c:activeNaivgators){
			c.getCommonViewer().refresh(true);
			c.getCommonViewer().refresh(c.getCommonViewer().getInput());
		}
	}

}
