/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.views.documents;

import org.w3c.dom.Node;

/**
 * Class representing a pair of node and corresponding location.
 * Comparison is performed by locations
 */
public class NodeLocation implements Comparable<NodeLocation>{
    
    protected LocationPosition location;
    protected Node node;
    
    public NodeLocation(final LocationPosition location, final Node node) {
        this.location = location;
        this.node = node;
    }

    public LocationPosition getLocation() {
        return location;
    }

    @Override
    public int compareTo(final NodeLocation other) {
        return location.compareTo(other.location);
    }
    
    @Override
    public String toString() {
    	String temp;
    	if (node.getNodeValue() == null) {
    		temp = "-";
    	} else {
    		temp = node.getNodeValue();
    	}
        return location.toString() + " tag:" + node.getNodeName() + " NodeType:" + node.getNodeType() 
        		+ " LocalName:" + node.getLocalName() + " value:<" + temp.replaceAll("\\n", "@").replaceAll("\\r", "~") + ">"; 
    }
    
    public Node getNode() {
        return node;
    }
    
}
