/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.TextViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.Comment;

public class CommentMainPropertySection extends TreeNodePropertiesTab {
    private Comment comment;

    private Text commentID, // Test Case Number
    			 commentName;
    private Text author;
    private TextViewer adesc;
    
    public CommentMainPropertySection() {
    }

    @Override
    public void createControls(Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        TabbedPropertySheetWidgetFactory wf = getWidgetFactory();

        parent.addFocusListener(new FocusListener() {
            @Override
            public void focusLost(FocusEvent e) {
            }

            @Override
            public void focusGained(FocusEvent e) {
            }
        });
        parent.setLayout(new GridLayout(2, false));
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));
        wf.createLabel(parent, "Id:").setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
        
        commentID = wf.createText(parent, "", SWT.BORDER);
        commentID.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        commentID.setEnabled(false);
        
        wf.createLabel(parent, "Name:").setLayoutData(
                new GridData(GridData.HORIZONTAL_ALIGN_END));
        
        commentName = wf.createText(parent, "");
        commentName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        commentName.addFocusListener(new FocusAdapter() {
        	public void focusLost(FocusEvent e) {
        		if(updating)
					return;
        		if (!commentName.getText().equals(commentName.getText().trim())) {
                    commentName.setText(commentName.getText().trim());
                }
                if (node == null) {
                    return;
                }
                if (node.getName() == null && commentName.getText().equals("") || 
                		node.getName() != null && 
                		node.getName().equals(commentName.getText())) {
                    return;
                }
               	if(node.getTreeDB().startTransaction("Update Requirement",true)){
               		node.setName(commentName.getText());
               		node.saveAttributes();
               		node.getTreeDB().commit();
        		}
               	else
               		commentName.setText(node.getName()!=null?node.getName():"");
            }
        });
        
        wf.createLabel(parent, "Author:").setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
        
        author = wf.createText(parent, "", SWT.BORDER);
        author.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        author.addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent e) {
            	if(updating)
					return;
                if(comment == null)
                    return;
                String newAuthor = author.getText().trim(),
                        old = comment.getStringAttributeValue(Comment.ATTR_AUTHOR);
                if (old != null && old.equals(newAuthor)) 
                    return;
                if(comment.getTreeDB().startTransaction("Update Comment Author",true)){
                	comment.putAttribute(new Attribute(comment, AttributeType.STRING, Comment.ATTR_AUTHOR, newAuthor));
                	comment.saveAttributes();
                	comment.getTreeDB().commit();
                }else
                	author.setText(old);
            }
        });
        GridData gridData = new GridData();
        gridData.verticalAlignment = GridData.FILL;
        gridData.horizontalSpan = 2;
        GridData gridData2 = new GridData(GridData.FILL_BOTH);
        gridData2.horizontalSpan = 2;
        
        adesc = new TextViewer(parent, SWT.MULTI | SWT.BORDER | SWT.H_SCROLL
                | SWT.V_SCROLL);
        adesc.getControl().setLayoutData(gridData2);
        adesc.getControl().addFocusListener(getDescriptionFocusAdapter(adesc, node));

    }

    @Override
    protected void setTreeNode(TreeNode node) {
        if (!(node instanceof Comment) || commentID == null
                || commentID.isDisposed()) {
            return;
        }
        comment = (Comment) node;
        commentID.setText(comment.getId());
        String auth = comment.getStringAttributeValue(Comment.ATTR_AUTHOR);
        author.setText(auth == null ? "" : auth);
        adesc.setDocument(new org.eclipse.jface.text.Document(comment
                .getDescription()));
        String text = comment.getName();
        if(text == null)
        	text = "";
        commentName.setText(text);
        
    }
}
