/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.WorkspaceJob;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.progress.UIJob;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.INodeChangeListener;
import com.unitesk.requality.core.ITreeNodeFilter;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.nodeiterators.RequalityChildIterator;
import com.unitesk.requality.core.nodeiterators.RequalityChildIterator.RequalityChildIteratorType;
import com.unitesk.requality.core.nodeiterators.ReuseRequalityChildIterator;
import com.unitesk.requality.core.transaction.TransactionStorage;
import com.unitesk.requality.eclipse.ui.dialogs.SelectNodeByTypesDialog;
import com.unitesk.requality.eclipse.ui.dialogs.SelectNodeDialog;
import com.unitesk.requality.nodetypes.Comment;
import com.unitesk.requality.nodetypes.VirtualNode;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.TestPurpose;

public class InstancerIteratorPropertySection extends TreeNodePropertiesTab {
    private VirtualNode instancer;
	private Composite parent;
	
    private static final String BUT_SEL = "Select...";
    static private String TITLE_SEL_BASE = "Select target node";
    private static final String BUT_CLEAR = "Clear...";
    static private String TITLE_CLEAR_BASE = "Clear target node";
    
    private Text target;
    private Font specFont;
    private Button selBaseButton, clearBaseButton;
    
    private List<Button>childGenVarsButtons = new ArrayList<Button>();
    private List<Combo>childGenVarsCombos = new ArrayList<Combo>();
    private Combo childGenList;
    private Button addchildGen;
    private Composite childGenVars;
	private Composite specPanel;
	private void updateSpecPanel(){
		if(isDisposed || specPanel.isDisposed())
			return;
		if(instancer == null || instancer.isDisposed())
			return;
		
		for(Control c: specPanel.getChildren())
    		c.dispose();
		Composite childGenVarsBase = null;
		if(instancer.getChildIterator().getChildIteratorType()==RequalityChildIteratorType.REUSE_CARTESIAN){
			Label l = new Label(specPanel,SWT.NONE);
	        l.setText("It. vars:");
	        l.setBackground(specPanel.getBackground());
	        l.setLayoutData(new GridData(SWT.RIGHT, SWT.TOP, false,false));
	        childGenVarsBase = new Composite(specPanel, SWT.NONE);
	        childGenVarsBase.setBackground(specPanel.getBackground());
	        childGenVarsBase.setLayoutData(new GridData(GridData.FILL_BOTH));
	        childGenVarsBase.setLayout(new GridLayout(1,true));
	        childGenVars = new Composite(childGenVarsBase, SWT.NONE);
	        childGenList.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
	        //childGenVars.setBackground(specPanel.getBackground());
			childGenVars.setLayout(new GridLayout(2,false));
			childGenVars.setBackground(specPanel.getBackground());
	    }
		updateChildGenList();
		parent.layout();
		if(instancer.getChildIterator().getChildIteratorType()==RequalityChildIteratorType.REUSE_CARTESIAN){
			addchildGen = new Button(childGenVarsBase, SWT.NONE);
			addchildGen.setText("+");
			addchildGen.setEnabled(((ReuseRequalityChildIterator)instancer.getChildIterator()).getAttributeNames(instancer.getTreeDB()).size()>0);
			addchildGen.setFont(specFont);
			GridData ld = new GridData();
			ld.heightHint = ld.widthHint = childGenList.getSize().y;
			addchildGen.setLayoutData(ld);
		}
		
		
		if(childGenVars!=null && !childGenVars.isDisposed())
			childGenVars.layout();
		specPanel.layout();

	}
	private void updateTargetUFName(final TreeNode nde){

        UIJob job = new UIJob("update requality cnf") {
            @Override
            public IStatus runInUIThread(IProgressMonitor monitor) {
            	if(instancer == null || instancer.getTarget() == null)
            		target.setText("<Target node is not selected>");
            	TreeNode node = nde;
            	if(node == null)
            		node = instancer.getTarget();
            	if((node == null || node.isDisposed()) &&!target.isDisposed())
            		target.setText("<Target node is not selected>");
            	if(!target.isDisposed() && node!=null)
            		target.setText(node.getUserFriendlyName());
				return Status.CANCEL_STATUS;
            }
        };
        job.shouldRun();
	}
	private void updateTargetUFName(){
		updateTargetUFName(null);
	}
    private INodeChangeListener iteratorChange = new INodeChangeListener() {
		@Override
		public void deleted(TreeNode source) {
	//		dispose();
		}
		
		@Override
		public void attributeChange(TreeNode source, String what) {
			final VirtualNode in = (VirtualNode)source;
			if(what.equals(VirtualNode.ATTR_CHILD_ITERATOR) || what.equals(TreeNode.ATTR_UNDEFINED)){
				InstancerIteratorPropertySection.this.getPart().getSite().getShell().getDisplay().asyncExec(new Runnable() {
					@Override
					public void run() {
						if(!specPanel.isDisposed()){
							childGenList.select(in.getChildIterator().getChildIteratorType().ordinal());
							updateChildGenList();
						}
					}
				});
			}else if (what.equals(VirtualNode.ATTR_TARGET) || what.equals(TreeNode.ATTR_UNDEFINED)){
				updateTargetUFName();
			}
				
		}
		@Override
		public void moved(TreeNode source, TreeNode oldParent,
				TreeNode newParent) {
			// TODO Auto-generated method stub
			
		}
	}, targetChange = new INodeChangeListener() {
		@Override
		public void deleted(TreeNode source) {
			if(instancer != null)
				updateTargetUFName(source);
		}
		
		@Override
		public void attributeChange(final TreeNode source, String what) {
			if(instancer == null)
				return;
			if(what.equals(TreeNode.ATTR_NAME) || what.equals(TreeNode.ATTR_UNDEFINED) || what.equals("_id")){
				WorkspaceJob wj = new WorkspaceJob("refresh name") {
					@Override
					public IStatus runInWorkspace(IProgressMonitor monitor)
							throws CoreException {
						updateTargetUFName(source);
						return Status.OK_STATUS;
					}
				};
				wj.schedule();
			}
		}
		@Override
		public void moved(TreeNode source, TreeNode oldParent,
				TreeNode newParent) {
			updateTargetUFName(source);
		}
	};
	
    public InstancerIteratorPropertySection() {}

    
    private Listener selBaseLink = new Listener() {
        public void handleEvent(Event event) {
        	try{
        		List<TreeNode>ignore = new ArrayList<TreeNode>();
        		if ((event.type == SWT.KeyDown) && (event.keyCode == SWT.INSERT)
	                    || (event.type == SWT.Selection)) {
	            	try {
		            	final Set<String> types = new HashSet<String>();
		            	types.add(Requirement.TYPE_NAME);
		            	types.add(TestPurpose.TYPE_NAME);
		            	types.add(Comment.TYPE_NAME);
		            	types.add(VirtualNode.TYPE_NAME);
		                SelectNodeDialog selDialog = new SelectNodeByTypesDialog(
		                		parent.getShell(), instancer.getTreeDB(), types);
		                selDialog.setFilter(new ITreeNodeFilter() {
							
							@Override
							public boolean select(TreeNode node) {
								if(!types.contains(node.getType()))
									return false;
								return instancer.getChildIterator().checkTarget(node)==null;
							}
						});
		                if (selDialog.open() == SelectNodeDialog.OK) {
		                    TreeNode selected = selDialog.getResult();
		                    if(selected!=null){
		                    	if(!node.getTreeDB().startTransaction("Select target node"))
		                    		return;
			            		setTarget(null);
		                    	setTarget(selected);
		                    	node.saveAttributes();
		                    	node.getTreeDB().commit();
		                    	return;
		                    }
		                }
					} catch (Exception e) {
						e.printStackTrace();
					}
        		}
            }
    		catch(Exception e){
    		}
        	node.getTreeDB().rollback();
        }
    },
    clearBaseLink = new Listener() {
        public void handleEvent(Event event) {
        	if(node.getTreeDB().startTransaction("Clear target node")){
        		setTarget(null);
        		node.saveAttributes();
        		node.getTreeDB().commit();
        	}
        }

		
    };
	private Map<Integer, FocusListener> lastComboSelectionListeners = new HashMap<Integer, FocusListener>();
	private Map<Integer, Listener> lastDelPressListeners = new HashMap<Integer, Listener>();
	private void setTarget(TreeNode selected) {
    	try {
    		TreeNode old = instancer.getTarget();
    		if(old!=null)
    			old.removeGuiChangeListener(targetChange);
    		instancer.hide = true;
    		String val = null;
    		if(selected!=null&&instancer.getChildIterator()!=null)
    			val = (instancer.getChildIterator().checkTarget(selected));
    		if(val!=null)
    		{
    			instancer.hide = false;
    			MessageDialog.openError(parent.getShell(),
                        "Target selection", "Selected node of type "+selected.getTypeLabel()+" cannot be child of "+instancer.getUserFriendlyName(true)+" because:\n"+val);
    			return;
    		}
    		instancer.hide = false;
    		//if(instancer.getTreeDB().startTransaction("Change target of reuse",true)){
    			try{
    				instancer.setTarget(selected);
    				instancer.saveAttributes();
    				//instancer.getTreeDB().commit();
    			}
    			catch(RuntimeException e){
    				MessageDialog.openError(parent.getShell(),
    						"Target selection", "Selected node cannot be target of "+instancer.getUserFriendlyName()+" because "+e.toString());
    			}
    		if(selected!=null)
    			selected.addGuiChangeListener(targetChange);
    		/*}else{
    			MessageDialog.openError(parent.getShell(),
    						"Target selection", "Some actions are already performing");
    			selected = instancer.getTarget();
    			if(selected != null)
    	    		target.setText(selected.getUserFriendlyName(true));
    	    	else
    	    		target.setText("");
    			return;
    		}*/
    			
		} catch (Exception e) {
			if(instancer.getTreeDB().getStorage() instanceof TransactionStorage)
				instancer.getTreeDB().rollback();
			e.printStackTrace();
		}
    	if(selected != null)
    		target.setText(selected.getUserFriendlyName(true));
    	else
    		target.setText("");
	}
	@Override
	public void dispose() {
		if(instancer != null){
			instancer.removeGuiChangeListener(iteratorChange);
			TreeNode nd = instancer.getTarget();
			if(nd!=null)
				nd.removeGuiChangeListener(targetChange);
		}
	};
	
    public Composite createComposite(Composite par, TabbedPropertySheetPage aTabbedPropertySheetPage){
    	parent = aTabbedPropertySheetPage.getWidgetFactory().createComposite(par);
        parent.setLayout(new GridLayout(1, true));
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        Composite panel = new Composite(parent, SWT.NONE);
        panel.setLayout(new GridLayout(3,false));
        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 2;
        panel.setLayoutData(gd);
        
        Label l = new Label(panel, SWT.NONE);
        l.setText("Target:");
        l.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
        l.setBackground(parent.getBackground());
        panel.setBackground(parent.getBackground());
        selBaseButton = new Button(panel, SWT.PUSH);
        selBaseButton.setText(BUT_SEL);
        selBaseButton.setToolTipText(TITLE_SEL_BASE);
        selBaseButton.addListener(SWT.Selection, selBaseLink);
        
        clearBaseButton = new Button(panel, SWT.PUSH);
        clearBaseButton.setText(BUT_CLEAR);
        clearBaseButton.setToolTipText(TITLE_CLEAR_BASE);
        clearBaseButton.addListener(SWT.Selection, clearBaseLink);

        GridLayout layout = new GridLayout(3, false);
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        
        target = new Text(parent, SWT.READ_ONLY);
        target.setText("");
        //target.setBackground(parent.getBackground());
        target.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        target.setBackground(parent.getBackground());
        
        Composite c = new Composite(parent,SWT.NONE);
        c.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        c.setLayout(new GridLayout(2,false));
        c.setBackground(parent.getBackground());
        l = new Label(c, SWT.NONE);
        l.setText("Iteration method:");
        l.setBackground(parent.getBackground());
        
        childGenList = new Combo(c, SWT.READ_ONLY);
		childGenList.add("Reuse");
		childGenList.add("Base element");
		childGenList.addModifyListener(new ModifyListener() {
			
			@Override
			public void modifyText(ModifyEvent e) {
				if(isDisposed || instancer.getTreeDB().getStorage() instanceof TransactionStorage)
					return;
				RequalityChildIterator ci = instancer.getChildIterator();
				if(ci == null || childGenList.getSelectionIndex() != -1 && 
						ci.getChildIteratorType().ordinal() != childGenList.getSelectionIndex()){
					int sel = childGenList.getSelectionIndex();
					if(sel < 0)
						sel = 0;
					RequalityChildIteratorType tp = RequalityChildIteratorType.values()[sel];
					if(instancer.getTreeDB().startTransaction("Change iterator type",true)){
						TreeNode targ = instancer.getTarget();
						try {
							instancer.setTarget(null);
							instancer.setChildIterator(RequalityChildIterator.fromAttributeString(null, instancer, tp.ordinal()+"|UUID:|"+Integer.toString(sel)));
							instancer.saveAttributes();
							instancer.setTarget(targ);
						} catch (Exception e1) {
							e1.printStackTrace();
						}
						instancer.getTreeDB().commit();
					}else
						childGenList.select(instancer.getChildIterator().getType().ordinal());
					updateSpecPanel();
				}	
			}
		});
		
        specPanel = new Composite(parent, SWT.NONE);
        specPanel.setLayoutData(new GridData(GridData.FILL_BOTH));
        specPanel.setLayout(new GridLayout(2, false));
        specPanel.setBackground(parent.getBackground());
        updateSpecPanel();
        return parent;
    }
    private String oldChildGenDescription = "";
    private void updateChildGenList() {
    	//childGenVarsCombos.clear();
		String newChildGenDesctiption = instancer.getStringAttributeValue(VirtualNode.ATTR_CHILD_ITERATOR);
		//if(oldChildGenDescription.equals(newChildGenDesctiption))
			//return;
		oldChildGenDescription = newChildGenDesctiption;
		if(newChildGenDesctiption == null)
			oldChildGenDescription = ""; 
		RequalityChildIterator rci = instancer.getChildIterator();
		if(rci != null){
			childGenList.select(rci.getChildIteratorType().ordinal());
		}else{
			childGenList.select(0);
			rci = instancer.getChildIterator();
		}
		if(rci != null && rci.getChildIteratorType() == RequalityChildIteratorType.REUSE_CARTESIAN){
			ReuseRequalityChildIterator ci = (ReuseRequalityChildIterator)rci;
			List<String> possibleVars = ci.getAttributeNames(instancer.getTreeDB()); 
			int i = 0;
					
			final List<String> vars = ci.getAttributeVars();
			while(vars.size()<childGenVarsCombos.size())
			{
				int ind = childGenVarsCombos.size();
				childGenVarsCombos.get(ind-1).dispose();
				childGenVarsCombos.remove(ind-1);
				childGenVarsButtons.get(ind-1).dispose();
				childGenVarsButtons.remove(ind-1);
				
			}
			for(String s : vars)
			{
				final int j = i;
				Combo c2 = null;
				Button b2 = null;
				if(j<childGenVarsCombos.size() && !childGenVarsCombos.get(j).isDisposed()){
					c2 = childGenVarsCombos.get(j);
					b2 = childGenVarsButtons.get(j);
					if(lastComboSelectionListeners.containsKey(j)){
						c2.removeFocusListener(lastComboSelectionListeners.get(j));
						b2.removeListener(SWT.MouseDown, lastDelPressListeners.get(j));
						c2.removeAll();
					}
				}else{
					c2 = new Combo(childGenVars, SWT.READ_ONLY);
					childGenVarsCombos.add(c2);
					//TODO check it
					//if(s.equals("") && !c2.getText().equals(""))
						//instancer.getChildIterator().setAttributeVar(i, c2.getText());
					b2 = new Button(childGenVars, SWT.NONE);
					b2.setFont(specFont);
					childGenVarsButtons.add(b2);

					GridData ld = new GridData();
					ld.heightHint = ld.widthHint = childGenList.getSize().y;
					b2.setLayoutData(ld);
					b2.setText("⊗");
				}
				final Combo c = c2;
				final Button b = b2;
				lastComboSelectionListeners.put(j, new FocusListener() {
					
					@Override
					public void focusLost(FocusEvent e) {
						if(e.widget.isDisposed()||instancer.getTreeDB().getStorage() instanceof TransactionStorage)
							return;
						if(((ReuseRequalityChildIterator) instancer.getChildIterator()).getAttributeVar(j)==c.getText())
							return;
						((Combo)e.widget).removeFocusListener(lastComboSelectionListeners.get(j));
						if(instancer.getTreeDB().startTransaction("Set parameter in iterator",true)){
							((ReuseRequalityChildIterator)instancer.getChildIterator()).setAttributeVar(instancer.getTreeDB(),j, c.getText());
							instancer.saveAttributes();
							instancer.getTreeDB().commit();
						}else
							((Combo)e.widget).select(((Combo)e.widget).indexOf(((ReuseRequalityChildIterator)instancer.getChildIterator()).getAttributeVar(j)));
						((Combo)e.widget).addFocusListener(lastComboSelectionListeners.get(j));		
					}
					
					@Override
					public void focusGained(FocusEvent e) {
						// TODO Auto-generated method stub
						
					}
				});
				lastDelPressListeners.put(j, new Listener() {
					@Override
					public void handleEvent(Event event) {
						if(event.widget.isDisposed()||instancer.getTreeDB().getStorage() instanceof TransactionStorage)
							return;
						if(instancer.getTreeDB().startTransaction("Remove parameter from iterator",true)){
							int id = childGenVarsButtons.indexOf(event.widget);
							((ReuseRequalityChildIterator)instancer.getChildIterator()).removeAttributeVar(instancer.getTreeDB(),id);
							event.widget.dispose();
							childGenVarsCombos.get(id).dispose();
							childGenVarsCombos.remove(id);
							childGenVarsButtons.remove(id);
							for(int i = id;i<childGenVarsCombos.size();i++){
								lastComboSelectionListeners.put(i, lastComboSelectionListeners.get(i+1));
							//	lastComboSelectionListeners.remove(i);
								lastDelPressListeners.put(i, lastDelPressListeners.get(i+1));
								//lastDelPressListeners.remove(i);
							}
							lastComboSelectionListeners.remove(childGenVarsCombos.size());
							lastDelPressListeners.remove(childGenVarsCombos.size());
							instancer.getTreeDB().commit();
						}
						updateChildGenList();
						childGenVars.layout();
						childGenVars.getParent().layout();
					}
			});
			int tosel = 0, k = 0;
			for(String s2:possibleVars){
				if(s2.startsWith("_"))
					continue;
				c.add(s2);
				if(s2.equals(s))
					tosel = k;
				k++;
			}
			c.select(tosel);
			c.addFocusListener(lastComboSelectionListeners.get(j));
			b.addListener(SWT.MouseDown, lastDelPressListeners.get(j));
			i++;
		}
		}
		//parent.layout();
		//specPanel.layout();
		
    	parent.layout();
    	parent.redraw();
		//if(childGenVars!=null && !childGenVars.isDisposed())
			//childGenVars.layout();

	}
	@Override
    public void createControls(final Composite par,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        par.setLayout(new GridLayout(1, true));
        par.setLayoutData(new GridData(GridData.FILL_BOTH));
        parent = createComposite(par, aTabbedPropertySheetPage);
        specFont = new Font(aTabbedPropertySheetPage.getSite().getShell().getDisplay(),java.awt.Font.MONOSPACED,14,0);
    }
	

    @Override
    protected void setTreeNode(TreeNode node) {
        if (!(node instanceof VirtualNode) || isDisposed || target.isDisposed()) {
            return;
        }
        
        instancer = (VirtualNode) node;
        if(isDisposed)
        	return;
        
        if(!target.isDisposed())
        	target.setText("");
        childGenVarsCombos.clear();
		childGenVarsButtons.clear();
		lastComboSelectionListeners.clear();
		lastDelPressListeners.clear();
        updateSpecPanel();
        
        switch(instancer.getChildIterator().getChildIteratorType()){
    	case REUSE_CARTESIAN:{
    				
    	}break;
    	case BASEREQ:{
    		
    	}}
       // if(instancer.getChildIterator()!=null)
       // 	iditer.setIterator(instancer.getChildIterator());
       // else{
       // 	instancer.setChildIterator((RequalityChildIterator) RequalityIterator.getOne(instancer, RequalityIteratorType.CHILD));
       // 	iditer.setIterator(instancer.getChildIterator()());
       // }
        if(instancer.getTarget() != null){
        	TreeNode targ = instancer.getTarget();
        	targ.removeGuiChangeListener(targetChange);
        	target.setText(targ.getUserFriendlyName());
        	targ.addGuiChangeListener(targetChange);
        }
        
        if(addchildGen!=null && !addchildGen.isDisposed()){
        	addchildGen.addListener(SWT.MouseDown, new Listener() {
				@Override
				public void handleEvent(Event event) {
					if(instancer.getTreeDB().startTransaction("Add parameter to iterator",true)){
						((ReuseRequalityChildIterator)instancer.getChildIterator()).addAttributeVar(instancer.getTreeDB());
						instancer.saveAttributes();
						instancer.getTreeDB().commit();
					}
					updateChildGenList();
					childGenVars.layout();
					childGenVars.getParent().layout();
				}
			});
        }
        
        node.addGuiChangeListener(iteratorChange);
        
        
    	parent.layout();
    	parent.redraw();
    }
}
