/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.ITreeLogic.IdContainsWrongChars;
import com.unitesk.requality.core.ITreeLogic.IdEmpty;

import com.unitesk.requality.core.ITreeLogic.SameIdExistsInChildren;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.VirtualNode;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.TestPurpose;

public class InstancerMainPropertySection extends TreeNodePropertiesTab {
    private VirtualNode instancer;
    private Text instancerID; // Test Case Number
    private Text instancerName; // Test Case Number
    private AttributesPanel attribPane; // Attributes
    private ControlDecoration testPurposeIDDecorator;

    public InstancerMainPropertySection() {
    }

    @Override
    public void createControls(Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        TabbedPropertySheetWidgetFactory wf = getWidgetFactory();

        parent.addFocusListener(new FocusListener() {
            @Override
            public void focusLost(FocusEvent e) {
            }

            @Override
            public void focusGained(FocusEvent e) {
            }
        });
        parent.setLayout(new GridLayout(2, false));
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));

        wf.createLabel(parent, "Id:").setLayoutData(
                new GridData(GridData.HORIZONTAL_ALIGN_END));

        instancerID = wf.createText(parent, "", SWT.BORDER);
        instancerID.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        testPurposeIDDecorator = createDecorator(instancerID);
        testPurposeIDDecorator.hide();
        instancerID.addModifyListener(new ModifyListener() {
            @Override
            public void modifyText(ModifyEvent e) {
                try {
                    if(instancer == null)
                        return;
                    if (instancer.getTreeDB().getTreeLogic().isIdValidExc(instancer, instancerID.getText().trim())) {
                        testPurposeIDDecorator.hide();
                    } else {
                        if (instancerID.getText().trim().equals("")) {
                            testPurposeIDDecorator
                                    .setDescriptionText("Virtual Node Id should not be empty");
                        } else {
                            testPurposeIDDecorator
                                    .setDescriptionText("Virtual Node Id is not correct or Virtual Node with the same Id already exists");
                        }
                        testPurposeIDDecorator.show();
                    }
                } catch (SameIdExistsInChildren e1) {
                    testPurposeIDDecorator
                        .setDescriptionText("Node with same Id already exists");
                } catch (IdEmpty e1) {
                    testPurposeIDDecorator
                        .setDescriptionText("Virtual Node Id is empty");
                } catch (IdContainsWrongChars e1) {
                    testPurposeIDDecorator
                        .setDescriptionText("Virtual Node Id contains wrong character - " + e1.getWrongCharacter());
                }
            }
        });
        instancerID.addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent e) {
            	if(updating)
					return;
            	if(instancer == null)
                    return;
                String newId = instancerID.getText().trim();
                instancerID.setText(newId);
                if (instancer.getId().equals(newId)) {
                    return;
                }
                if (!instancer.getQualifiedId().equals(
                        Requirement.getTypeRootQId())) {
                    if (instancer.getTreeDB().getTreeLogic().isIdValid(instancer, newId)&&instancer.getTreeDB().startTransaction(
                               "Update Virtual Node",true)) {
                   		instancer.setId(newId);
                   		instancer.getTreeDB().commit();
                    } else {
                        instancerID.setText(instancer.getId());
                    }
                }
            }
        });
        
        wf.createLabel(parent, "Name:").setLayoutData(
                new GridData(GridData.HORIZONTAL_ALIGN_END));
        
        instancerName = wf.createText(parent, "");
        instancerName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        instancerName.addFocusListener(new FocusAdapter() {
        	public void focusLost(FocusEvent e) {
        		if(updating)
					return;
        		if (!instancerName.getText().equals(instancerName.getText().trim())) {
                    instancerName.setText(instancerName.getText().trim());
                }
                if (node == null) {
                    return;
                }
                if (node.getName() == null && instancerName.getText().equals("") || 
                		node.getName() != null && 
                		node.getName().equals(instancerName.getText())) {
                    return;
                }
                if(node.getTreeDB().startTransaction("Update Requirement",true)){
                	node.setName(instancerName.getText());
                	node.saveAttributes();
                	node.getTreeDB().commit();
                }else
                	instancerName.setText(node.getName()!=null?node.getName():"");
            }
        });
        
        attribPane = new AttributesPanel(parent, wf);
        GridData gridData = new GridData(GridData.FILL_BOTH);
        gridData.horizontalSpan = 2;
        attribPane.getControl().setLayoutData(gridData);
    }

    private ControlDecoration createDecorator(Text text) {
        ControlDecoration controlDecoration = new ControlDecoration(text,
                SWT.LEFT | SWT.TOP);
        FieldDecoration fieldDecoration = FieldDecorationRegistry.getDefault()
                .getFieldDecoration(FieldDecorationRegistry.DEC_ERROR);
        controlDecoration.setImage(fieldDecoration.getImage());
        return controlDecoration;
    }

    @Override
    protected void setTreeNode(TreeNode node) {
        if (!(node instanceof VirtualNode) || isDisposed) {
            return; 
        }
        instancer = (VirtualNode) node;
        instancerID.setText(instancer.getId());
        String text = instancer.getName();
        if(text == null)
        	text = "";
        instancerName.setText(text);
        attribPane.setNode(instancer);
    }
}
