/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.nodetypes;

import java.util.UUID;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;


public class NodeType extends TreeNode {
    public static final String TYPE_NAME = "NodeType";
    private static final String ATTRIBUTE_CLASSNAME = "_classname";
    private static final String ATTRIBUTE_ICON = "_iconResource";
    private Class<? extends TreeNode> clazz;
    public NodeType(String name, TreeDB db) {
        super(db, UUID.randomUUID());
        this.id = name;
        setType(TYPE_NAME);
    }

    public NodeType(TreeDB db, UUID uuid) {
        super(db, uuid);
    }

    public String getName() {
        return getId();
    }

    @Override
    public String toString() {
        return "TypeNode '" + getName() + "' (" + getUUId() + ")";
    }

    public void setClassName(String className) {
        putAttribute(new Attribute(this, AttributeType.STRING, ATTRIBUTE_CLASSNAME, className));
    }

    public void setClass(Class<? extends TreeNode> clazz) {
        putAttribute(new Attribute(this, AttributeType.STRING, ATTRIBUTE_CLASSNAME, clazz.getName()));
        this.clazz=clazz;
    }

    @SuppressWarnings("unchecked")
    public Class<? extends TreeNode> getTypeClass() {
        if(clazz!=null) {
            return clazz;
        }
        String className = (String) getAttributeValue(ATTRIBUTE_CLASSNAME);
        if(className==null) {
            return null;
        }
        try {
            return (Class<? extends TreeNode>) Class.forName(className);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
        return null;
    }

    public String getIconName() {
        if (getAttributeValue(ATTRIBUTE_ICON) == null)
            return getId();
        return (String) getAttributeValue(ATTRIBUTE_ICON);
    }

    public void setName(String icon) {
        putAttribute(new Attribute(this ,AttributeType.STRING, ATTRIBUTE_ICON, icon));
    }

    @Override
    public boolean equals(Object obj) {
        if( super.equals(obj) ) return true;
        if(obj instanceof NodeType) {
            NodeType t = (NodeType) obj;
            if(t.getName().equals(getName())) return true;
        }
        return false;
    }
    
}
