package com.unitesk.requality.report;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.UUID;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.Platform;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.unitesk.requality.core.FSStorage;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.cvs.CVSManager;
import com.unitesk.requality.eclipse.cvs.FileSpaceTimePoint;
import com.unitesk.requality.nodetypes.NodeType;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.TestPurpose;
import com.unitesk.requality.tests.CoverageStorage;

public class StatisticManager {
	private IProject project;
	IProject[] cvnProjects;
	File[] copyProjects;
	private String filePath;
	private Requirement req;
	private CoverageStorage cs;
	private CoverageStorage basecs;
	private FileSpaceTimePoint svnInfo;
	private String error = null;

	private ArrayList<String> revisionList = new ArrayList<String>();
	private ArrayList<String> revEmptyList = new ArrayList<String>();
	private String speshRev = "";
	private Long dateFrom = Long.MIN_VALUE, dateTo = Long.MAX_VALUE;

	private Integer rq = 0;
	private Integer tp = 0;
	private Integer leaves = 0;
	private Map<String, Integer> covCount = new HashMap<String, Integer>();
	private List<UUID> calculated = new ArrayList<UUID>();

	public StatisticManager(CoverageStorage cs2, IProject project, String path) {
		fstpPoints.clear();
		this.project = project;
		error = null;
		basecs = cs2;
		basecs.loadStorage();
		this.filePath = path.substring(5, path.length() - 11) + File.separator
				+ "statistics_"
				+ basecs.getCoverageSource().getClass().getSimpleName()
				+ ".xml";
		Object o = basecs.getCoverageSourceObject();
		// TODO in case of TreeNodes - load related projects
		File[] linkedProjects = null;
		try {
			if (o instanceof File
					&& !(((File) o).getCanonicalPath().equals(project
							.getLocation().toOSString())))
				linkedProjects = new File[] { (File) o };
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		if (o instanceof File[]) {
			List<File> list = new ArrayList<File>();
			for (File cn : (File[]) o) {
				try {
					if (!(((File) cn).getCanonicalPath().equals(project
							.getLocation().toOSString())))
						list.add(cn);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			linkedProjects = list.toArray(new File[0]);
		}
		List<IProject> cvn = new ArrayList<IProject>();
		List<File> ncvn = new ArrayList<File>();
		if (linkedProjects != null)
			for (File cn : linkedProjects) {
				CVSManager cvsm = Activator.getDefault().getCVSManager();
				boolean cvned = false;
				try {
					// Hack
					String p1 = cn.getCanonicalPath(), p2 = Platform
							.getLocation().toOSString();
					int c = 0;
					while (c < p1.length() && c < p2.length()
							&& p1.charAt(c) == p2.charAt(c))
						c++;
					if (c == p2.length()) {
						p1 = p1.substring(c + 1);
						IProject proj = Activator.getProjectByName(p1
								.substring(0, p1.indexOf(File.separator)));
						if (proj != null) {
							cvsm.getCVSClient(proj);
							cvn.add(proj);
							cvned = true;
						}
					}
				} catch (Exception e) {
				}
				if (!cvned)
					ncvn.add(cn);
			}
		cvnProjects = cvn.toArray(new IProject[0]);
		copyProjects = ncvn.toArray(new File[0]);
	}

	public String getErrorMsg() {
		return error;
	}

	public String getPath() {
		return basecs.getCoverageSource().getClass().getSimpleName();
		// Arrays.toString(basecs.getSettings(rootReq))+" "+basecs.getCoverageSource().getClass().getSimpleName());
	}

	public boolean isDateCorrect(String dateAfter, String dateBefore) {
		SimpleDateFormat format = new SimpleDateFormat(
				"EEE MMM dd HH:mm:ss zzz yyyy", Locale.ENGLISH);
		try {
			if ((dateAfter != null) && (!dateAfter.isEmpty())) {
				this.dateFrom = format.parse(dateAfter).getTime();
			}
			if ((dateBefore != null) && (!dateBefore.isEmpty())) {
				this.dateTo = format.parse(dateBefore).getTime();
			}
			if ((dateAfter != null) && (!dateAfter.isEmpty())
					&& (dateBefore != null) && (!dateBefore.isEmpty())) {
				if (this.dateTo - this.dateFrom <= 0) {
					System.out
							.println("Wrong dates. Date 'from' can't be equals or after date 'to'.");
					return false;
				}
			}
		} catch (ParseException e) {
			System.out.println("Wrong date format.");
			return false;
		}
		return true;
	}

	public void refreshRecordsList() {
		try {
			CVSManager cvsm = Activator.getDefault().getCVSManager();
			Set<FileSpaceTimePoint> ps = new HashSet<FileSpaceTimePoint>(
					cvsm.getRevisions(((IResource) this.project)));
			for (FileSpaceTimePoint pp : ps) {
				writeHeaderReport(pp);
			}
			if (cvsm.getLastRevError() != null)
				error = cvsm.getLastRevError();
		} catch (Exception e) {
			error = e.toString();
			System.out.println(e.toString());
		}
	}

	public boolean isInfoExist() {
		this.revisionList = new ArrayList<String>();
		this.revEmptyList = new ArrayList<String>();
		this.speshRev = "";
		Boolean next = true;
		try {
			File file = new File(filePath);
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document doc = dBuilder.parse(file);
			doc.getDocumentElement().normalize();
			NodeList nlist = doc.getElementsByTagName("statistic");

			SimpleDateFormat format = new SimpleDateFormat(
					"EEE MMM dd HH:mm:ss zzz yyyy", Locale.ENGLISH);
			int rev = -1;
			for (int s = nlist.getLength() - 1; s >= 0; s--) {
				Node node = nlist.item(s);
				if (node.getNodeType() == Node.ELEMENT_NODE) {
					Element elm = (Element) node;
					Long attTime = format.parse(elm.getAttribute("time"))
							.getTime();
					if (this.dateFrom == Long.MIN_VALUE
							&& this.dateTo == Long.MAX_VALUE) {
						if (!node.hasChildNodes()
								&& !elm.hasAttribute("noChild")) {
							revisionList.add(elm.getAttribute("numRevision"));
						}
					} else if ((this.dateFrom <= attTime)
							&& (this.dateTo >= attTime)) {
						rev = s;
						if (!node.hasChildNodes()
								&& !elm.hasAttribute("noChild")) {
							revisionList.add(elm.getAttribute("numRevision"));
						}
					} else if (this.dateFrom > attTime && next) {
						if (!node.hasChildNodes()
								&& !elm.hasAttribute("noChild")) {
							revEmptyList.add(elm.getAttribute("numRevision"));
						} else {
							next = false;
						}
					}
				}
			}
			rev--;
			if (rev >= 0) {
				Node nodePrev = nlist.item(rev);
				if (nodePrev.getNodeType() == Node.ELEMENT_NODE) {
					Element elmPrev = (Element) nodePrev;
					if (!nodePrev.hasChildNodes()) {
						speshRev = elmPrev.getAttribute("numRevision");
					}
				}
			}
			return (this.revisionList.isEmpty() && speshRev.equals(""));
		} catch (ParseException e) {
			System.out.println("Wrong date format.");
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (IOException e) {
			System.out.println("File doesn't exist.");
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}

	public void getDateRecord() {
		CVSManager cvsm = Activator.getDefault().getCVSManager();
		Set<FileSpaceTimePoint> ps = new HashSet<FileSpaceTimePoint>(
				cvsm.getRevisions(((IResource) this.project)));
		List<FileSpaceTimePoint> revList = getRevisions(cvsm, ps);
		for (FileSpaceTimePoint rev : revList) {
			getRevisionInfo(this.project, cvsm, rev);
		}
		if (!speshRev.equals("")) {
			getPrevRevision(cvsm, ps);
		}
	}

	public List<FileSpaceTimePoint> getRevisions(CVSManager cvsm,
			Set<FileSpaceTimePoint> ps) {
		Map<String, FileSpaceTimePoint> map = new HashMap<String, FileSpaceTimePoint>();
		for (FileSpaceTimePoint pp : ps) {
			if (this.revisionList.contains(pp.getRevision()))
				map.put(pp.getRevision(), pp);
		}
		SortedSet<Integer> sortedRev = new TreeSet<Integer>();
		Map<Integer, String> ii = new HashMap<Integer, String>();
		for (String s : map.keySet()) {
			ii.put(Integer.parseInt(s), s);
			sortedRev.add(Integer.parseInt(s));// TODO
		}
		List<FileSpaceTimePoint> l = new LinkedList<FileSpaceTimePoint>();
		for (Integer i : sortedRev)
			l.add(map.get(ii.get(i)));
		return l;
	}

	public void getRevisionInfo(IProject proj, CVSManager cvsm,
			FileSpaceTimePoint pp) {
		try {
			TreeDB odb = TreesTracker.getTracker().getTree((IProject) proj);

			java.io.File f = cvsm
					.getCVSClient(((IResource) proj).getProject())
					.getContainerFromRepoS(pp.getPath(), pp.getRevision(), null);
			FSStorage fs = new FSStorage(f.getAbsolutePath() + "/");

			TreeDB db = TreeDB.load(fs);
			for (NodeType nt : odb.getTypes().values()) {
				db.addNodeType(nt.getTypeClass());
			}

			loadRelatedProjects(cvsm, f.getParentFile().getParentFile()
					.getAbsolutePath(), pp);

			CoverageStorage cs = getNewCoverage(basecs, db);
			try {
				createNewRecord(
						(Requirement) db.getNode(Requirement.getTypeRootQId()),
						cs, pp);
			} catch (NullPointerException e) {
				writeNoContentReport(pp.getRevision());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private Map<IProject, FileSpaceTimePoint[]> fstpPoints = new HashMap<IProject, FileSpaceTimePoint[]>();

	private void loadRelatedProjects(CVSManager cvsm, String absolutePath,
			FileSpaceTimePoint pp) {
		for (File c : copyProjects)
			copy(c, absolutePath);

		for (IProject c : cvnProjects) {
			if (!fstpPoints.containsKey(c))
				fstpPoints
						.put(c,
								cvsm.getRevisions(c).toArray(
										new FileSpaceTimePoint[0]));
			FileSpaceTimePoint[] points = fstpPoints.get(c);
			int mn = 0, mx = 0;
			for (int i = 0; i < points.length; i++) {
				FileSpaceTimePoint tp = points[i];
				if ((points[mx].getEpochTimeSec() < pp.getEpochTimeSec() || tp
						.getEpochTimeSec() < points[mx].getEpochTimeSec())
						&& tp.getEpochTimeSec() > pp.getEpochTimeSec())
					mx = i;
				if ((points[mn].getEpochTimeSec() > pp.getEpochTimeSec() || tp
						.getEpochTimeSec() > points[mn].getEpochTimeSec())
						&& tp.getEpochTimeSec() < pp.getEpochTimeSec())
					mn = i;
			}
			FileSpaceTimePoint sel = points[mn];
			try {
				copy(cvsm.getCVSClient(c).getContainerFromRepoS(sel.getPath(),
						sel.getRevision(), null), absolutePath);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	private void getPrevRevision(CVSManager cvsm, Set<FileSpaceTimePoint> ps) {
		FileSpaceTimePoint rev = null;
		for (FileSpaceTimePoint pp : ps) {
			if (pp.getRevision().equals(this.speshRev)) {
				rev = pp;
				break;
			}
		}
		Boolean next = getPrevRevisionInfo(this.project, rev);
		Integer revNum = Integer.parseInt(speshRev) - 1;

		while ((!next) && (revNum > 0)) {
			speshRev = revNum.toString();
			if (revEmptyList.contains(speshRev)) {
				for (FileSpaceTimePoint pp : ps) {
					if (pp.getRevision().equals(this.speshRev)) {
						rev = pp;
						break;
					}
				}
				next = getPrevRevisionInfo(this.project, rev);
			}
			revNum--;
		}
	}

	private static TreeDB db;

	public boolean getPrevRevisionInfo(IProject proj, FileSpaceTimePoint pp) {
		try {
			CVSManager cvsm = Activator.getDefault().getCVSManager();
			TreeDB odb = TreesTracker.getTracker().getTree((IProject) proj);

			// if(odb == null)
			// continue;
			// получение всей полной инфы
			java.io.File f = cvsm
					.getCVSClient(((IResource) proj).getProject())
					.getContainerFromRepoS(pp.getPath(), pp.getRevision(), null);
			FSStorage fs = new FSStorage(f.getAbsolutePath() + "/");

			db = TreeDB.load(fs);
			for (NodeType nt : odb.getTypes().values()) {
				db.addNodeType(nt.getTypeClass());
			}

			loadRelatedProjects(cvsm, f.getParentFile().getParentFile()
					.getAbsolutePath(), pp);

			CoverageStorage cs = getNewCoverage(basecs, db);
			try {
				createNewRecord(
						(Requirement) db.getNode(Requirement.getTypeRootQId()),
						cs, pp);
				return true;
			} catch (NullPointerException e) {
				writeNoContentReport(pp.getRevision());
				return false;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}

	private void copy(File c, String absolutePath) {
		try {
			File f = c;
			List<File> hndl = new ArrayList<File>();
			List<String> path = new ArrayList<String>();
			int i = 0;
			hndl.add(f);
			path.add(absolutePath);
			while (hndl.size() > i) {
				File t = hndl.get(i);
				String g = path.get(i) + File.separator + t.getName();
				if (t.isDirectory()) {
					(new File(g)).mkdir();
					for (File f1 : t.listFiles()) {
						hndl.add(f1);
						path.add(g);
					}
				} else {
					File cf = (new File(g));
					cf.createNewFile();
					FileReader fr = new FileReader(t);
					FileWriter fw = new FileWriter(cf);
					while (fr.ready())
						fw.write(fr.read());
					fw.close();
					fr.close();
				}
				i++;
			}
			// c.getFolder(new Path("./")).copy(new Path(absolutePath), true,
			// null);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private CoverageStorage getNewCoverage(CoverageStorage base, TreeDB db) {
		CoverageStorage cs = CoverageStorage.getStorage(db,
				base.getCoverageSource());
		TreeNode node = null;
		try {
			node = db.getNode(Requirement.getTypeRootQId());
		} catch (Exception e) {

		}
		if (node != null) {
			cs.getCoverageSource().setSettings(base.getSettings(node));
			cs.loadStorage();
			return cs;
		}
		return base;
	}

	public Map<String, Integer> getArrayData(Requirement rootReq,
			CoverageStorage cs) {
		createNewRecord(rootReq, cs, null);
		Map<String, Integer> data = new HashMap<String, Integer>();
		data.put("number of requirements", rq);
		data.put("number of test purposes", tp);
		data.put("number of leaves", leaves);
		data.put("number coverage (green status)", covCount.get("green"));
		data.put("number coverage (red status)", covCount.get("red"));
		float sum = covCount.get("green") + covCount.get("red");
		data.put("percent coverage (green status)",
				Math.round((float) covCount.get("green") * 100 / sum));
		data.put("percent coverage (red status)",
				Math.round((float) covCount.get("red") * 100 / sum));
		return data;
	}

	private void createNewRecord(Requirement rootReq, CoverageStorage cs,
			FileSpaceTimePoint svnInfo) {
		cs.loadStorage();
		req = rootReq;
		this.cs = cs;
		this.svnInfo = svnInfo;
		rq = 0;
		tp = 0;
		leaves = 0;
		covCount.put("green", 0);
		covCount.put("red", 0);
		calculated = new ArrayList<UUID>();

		int sc = req.sizeChildren();
		if (req.getType().equals(Requirement.TYPE_NAME))
			rq += 1;
		for (int i = 0; i < sc; i++) {
			TreeNode node = req.getChild(i);
			this.calculateStatistics(node);
		}
		if (svnInfo != null) {
			writeContentReport();
		}
	}

	private void calculateStatistics(TreeNode sc) {
		if (calculated.contains(sc.getUUId()))
			return;
		// check node is requirement
		if (sc.getType().equals(Requirement.TYPE_NAME))
			rq += 1;
		// check node is test purpose
		if (sc.getType().equals(TestPurpose.TYPE_NAME))
			tp += 1;
		if (sc.sizeChildren() == 0) {
			leaves += 1;
			// check if node is directly covered by test
			if (cs.isCovered(sc.getQualifiedId())) {
				covCount.put("green", covCount.get("green") + 1);
			} else {
				covCount.put("red", covCount.get("red") + 1);
			}
		}
		// get first test path that covers node
		// String test =
		// cs.getCoverageInfo(node.getQualifiedId()).getTestPaths()[0];
		// cs.getErrorsForPath(test);
		calculated.add(sc.getUUId());
		for (int i = 0; i < sc.sizeChildren(); i++) {
			TreeNode node = sc.getChild(i);
			this.calculateStatistics(node);
		}
	}

	DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
	DocumentBuilder dBuilder = null;

	private void writeHeaderReport(FileSpaceTimePoint pp) {
		try {
			File file = new File(filePath);
			if (dBuilder == null)
				dBuilder = dbFactory.newDocumentBuilder();
			Document doc;

			if (file.exists()) {
				doc = dBuilder.parse(file);
				doc.getDocumentElement().normalize();
				boolean insert = false;
				Integer rev = Integer.parseInt(pp.getRevision());
				NodeList statList = doc.getElementsByTagName("statistic");
				for (int s = 0; s < statList.getLength(); s++) {
					Node statNode = statList.item(s);
					if (statNode.getNodeType() == Node.ELEMENT_NODE) {
						Element elm = (Element) statNode;
						Integer attRev = Integer.parseInt(elm
								.getAttribute("numRevision"));
						if (rev == attRev) {
							insert = true;
							break;
						}
						if (rev < attRev) {
							Element newChild = createHeaderElement(doc, pp);
							elm.getParentNode().insertBefore(newChild, elm);
							insert = true;
							break;
						}
					}
				}
				if (!insert) {
					Node stat = doc.getElementsByTagName("statistics").item(0);
					Element newChild = createHeaderElement(doc, pp);
					stat.appendChild(newChild);
				}
			} else {
				doc = dBuilder.newDocument();
				Element rootlvl1 = doc.createElement("statistics");
				doc.appendChild(rootlvl1);
				Element rootlvl2 = createHeaderElement(doc, pp);
				rootlvl1.appendChild(rootlvl2);
				file.getParentFile().mkdirs();
				file.createNewFile();
			}
			TransformerFactory tfFactory = TransformerFactory.newInstance();
			Transformer transformer = tfFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
			StreamResult strRes = new StreamResult(file);
			transformer.transform(source, strRes);
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (TransformerException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private Element createHeaderElement(Document doc, FileSpaceTimePoint pp) {
		Element newChild = doc.createElement("statistic");
		if (pp != null) {
			newChild.setAttribute("time",
					new Date(pp.getEpochTimeSec()).toString());
			newChild.setAttribute("author", pp.getAuthor());
			newChild.setAttribute("numRevision", pp.getRevision());
		} else {
			newChild.setAttribute("time", new Date().toString());
			newChild.setAttribute("author", System.getProperty("user.name"));
			newChild.setAttribute("numRevision", "0");
		}
		return newChild;
	}

	private void writeContentReport() {
		try {
			File file = new File(filePath);
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document doc;

			if (file.exists()) {
				doc = dBuilder.parse(file);
				doc.getDocumentElement().normalize();
				String rev = svnInfo.getRevision();
				NodeList statList = doc.getElementsByTagName("statistic");
				for (int s = 0; s < statList.getLength(); s++) {
					Node statNode = statList.item(s);
					if (statNode.getNodeType() == Node.ELEMENT_NODE) {
						Element elm = (Element) statNode;
						if (rev.equals(elm.getAttribute("numRevision"))) {
							Element node = doc.createElement("reqCount");
							node.setAttribute("name", "number of requirements");
							node.setAttribute("value", rq.toString());
							elm.appendChild(node);

							node = doc.createElement("tpCount");
							node.setAttribute("name", "number of test purposes");
							node.setAttribute("value", tp.toString());
							elm.appendChild(node);

							node = doc.createElement("leaves");
							node.setAttribute("name", "leaves");
							node.setAttribute("value", leaves.toString());
							elm.appendChild(node);

							node = doc.createElement("statuses");
							for (Map.Entry<String, Integer> entry : covCount
									.entrySet()) {
								Element nodelvl2 = doc.createElement("status");
								nodelvl2.setAttribute("name", entry.getKey());
								nodelvl2.setAttribute("value", entry.getValue()
										.toString());
								node.appendChild(nodelvl2);
							}
							elm.appendChild(node);
						}
					}
				}
				TransformerFactory tfFactory = TransformerFactory.newInstance();
				Transformer transformer = tfFactory.newTransformer();
				DOMSource source = new DOMSource(doc);
				StreamResult strRes = new StreamResult(file);
				transformer.transform(source, strRes);
			} else {
				System.out.println("File" + filePath + "doesn't exist!");
			}
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (TransformerException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void writeNoContentReport(String revNum) {
		try {
			File file = new File(filePath);
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document doc;

			if (file.exists()) {
				doc = dBuilder.parse(file);
				doc.getDocumentElement().normalize();

				NodeList statList = doc.getElementsByTagName("statistic");
				for (int s = 0; s < statList.getLength(); s++) {
					Node statNode = statList.item(s);
					if (statNode.getNodeType() == Node.ELEMENT_NODE) {
						Element elm = (Element) statNode;
						if (revNum.equals(elm.getAttribute("numRevision"))) {
							elm.setAttribute("noChild", "true");
						}
					}
				}
				TransformerFactory tfFactory = TransformerFactory.newInstance();
				Transformer transformer = tfFactory.newTransformer();
				DOMSource source = new DOMSource(doc);
				StreamResult strRes = new StreamResult(file);
				transformer.transform(source, strRes);
			} else {
				System.out.println("File" + filePath + "doesn't exist!");
			}
		} catch (TransformerException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
