package com.unitesk.requality.tests;


import java.util.List;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ElementListSelectionDialog;
import org.eclipse.ui.dialogs.ListSelectionDialog;
import org.eclipse.ui.dialogs.SaveAsDialog;
import org.eclipse.ui.wizards.IWizardDescriptor;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.TestGeneratorDescriptor;
import com.unitesk.requality.eclipse.wizards.ITestGenWizard;

public class TestDialogProvider {
	private Shell shell;
	private TreeDB db;
	public TestDialogProvider(Shell s, TreeDB db){
		shell = s;
		this.db = db;
	}
    private static TestDialogProvider testDialog;
	private static TestDialogProvider getTestDialogProvider(TreeDB db){
    	if (testDialog == null)
    		testDialog = new TestDialogProvider(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), db);
    	return testDialog;
    }

	public static boolean fireTestCreation(TreeDB db, String qid, Shell shell2){
		TreeNode target = null;
        try{
        	target = db.getNode(qid);
        }catch(Exception e){
        	return false;
        }
        
        SortedSet<TestGeneratorDescriptor> set = Activator.getDefault().getTestGenerators(),
        								   fset;
        List<String> filters = db.getTestGeneratorsFilters();
    	fset = new TreeSet<TestGeneratorDescriptor>();
        if(filters != null){
        	for(TestGeneratorDescriptor tgd:set)
        		if(filters.contains(tgd.getId()) && tgd.getGenerator().isEnabled(target))
        			fset.add(tgd);
        }else
        	for(TestGeneratorDescriptor tgd:set)
        		if(tgd.getGenerator().isEnabled(target))
        			fset.add(tgd);
        
        TestGeneratorDescriptor choosed;
        
        if(fset.size() == 0){
        	MessageDialog.openInformation(shell2, "Test generators not available", "There is no available test generators. Please install plugins with test generators");
        	return false;
        }else{
        	choosed = fset.first();
        	if (fset.size() > 1){
        		choosed = fireSelectionFromList(fset);
        	}
        }
        if(choosed == null)
        	return false;
        ITestGenerator gen = choosed.getGenerator();
        if (gen == null)
        		return false;
        return gen.generateTest(target, getTestDialogProvider(db));
	}
	
	public static TestGeneratorDescriptor fireSelectionFromList(Set<TestGeneratorDescriptor> gens){
    	ElementListSelectionDialog dialog = new ElementListSelectionDialog(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), new LabelProvider());

    	dialog.setTitle("Test Generator Selection");

    	dialog.setMessage("Select a String (* = any string, ? = any char):");

    	dialog.setElements(gens.toArray());

    	int res = dialog.open();
    	if(res == ListSelectionDialog.OK)
    		return (TestGeneratorDescriptor)dialog.getResult()[0];
    	return null;
	}
	
	public OperationResult openCreateFileInEclipseProjectDialog(String extenstion, String title){
		SaveAsDialog sad = new SaveAsDialog(shell);
		sad.setTitle(title);
		int res = sad.open();

		return new OperationResult(sad.getResult().toFile().getAbsolutePath()+"."+extenstion, res == SaveAsDialog.OK);
	}
	public OperationResult openCreateFileDialog(String extenstion, String title){
		FileDialog dialog = new FileDialog (shell, SWT.SAVE);
		dialog.setText(title);
		String [] filterNames = new String [] {extenstion+" files", "All Files (*)"};
		String [] filterExtensions = new String [] {"*."+extenstion, "*"};
		
		dialog.setFilterNames (filterNames);
		dialog.setFilterExtensions (filterExtensions);
		
		String res = dialog.open();

		return new OperationResult(res == null ? "" : res, res != null);
	}

	public OperationResult openWizard(String wizardid, TreeNode target) {
		return openWizard(null, wizardid, target);		
	}
	public OperationResult openWizard(WizardDialog wd, String wizardid, TreeNode target) {
		if(wd == null)
			wd = getWizard(wizardid, target);
		boolean succeed = false;
		String message = null;
		try{
     		succeed = wd.open() == WizardDialog.OK;
     	}
     	catch(Exception e){
     		e.printStackTrace();
     	}
     	//TODO check
     	message = ((ITestGenWizard)wd.getCurrentPage().getWizard()).getTargetQID();
		return new OperationResult(message, succeed);		
	}
	
	public WizardDialog getWizard(String wizardId, TreeNode targetNode) {
		String message = "";
		boolean succeed = false;
		// First see if this is a "new wizard".
		IWizardDescriptor descriptor = PlatformUI.getWorkbench()
			.getNewWizardRegistry().findWizard(wizardId);
			 // If not check if it is an "import wizard".
		if  (descriptor == null) {
			descriptor = PlatformUI.getWorkbench().getImportWizardRegistry()
				.findWizard(wizardId);
		}
		// Or maybe an export wizard
 		if  (descriptor == null) {
   			descriptor = PlatformUI.getWorkbench().getExportWizardRegistry()
   				.findWizard(wizardId);
 		}
 		try  {
		   // 	Then if we have a wizard, open it.
   			if  (descriptor != null) {
     			wizard = descriptor.createWizard();
     			if(wizard instanceof ITestGenWizard)
     			{
     				WizardDialog wd = new  WizardDialog(shell, wizard);
     				wd.setTitle(wizard.getWindowTitle());
     				((ITestGenWizard)wizard).setTreeDB(targetNode.getTreeDB());
     				((ITestGenWizard)wizard).setTargetQID(targetNode.getUserFriendlyName(),false);
     				return wd;
     			}
   			}
 		} catch  (CoreException e) {
 		 	message = e.toString();
 		}
		return null;	
	}
	private IWizard wizard;
	public IWizard getLastWizard() {
		return wizard;
	}
}

