package com.unitesk.requality.tools;

import java.io.IOException;
import java.io.StringWriter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.wizards.NewRequalityProjectWizard;
import com.unitesk.requality.nodetypes.Requirement;

public class ReqIFConverter {
	public static Set<String> loadReqIFClasses(Document model){
		Set<String> classes = new HashSet<String>();
		Node n = model.getChildNodes().item(0);
		n = getFirstChild(model, "REQ-IF");
		if(n==null)
			return null;
		n = getFirstChild(n, "CORE-CONTENT");
		n = getFirstChild(n, "REQ-IF-CONTENT");
		n = getFirstChild(n, "SPEC-TYPES");
		for(int j = 0;j<n.getChildNodes().getLength();j++){
			Node node = n.getChildNodes().item(j);
			if(node.getNodeName().equals("SPEC-OBJECT-TYPE")){
				Node nd = node.getAttributes().getNamedItem("LONG-NAME");
				if(nd!=null)
					classes.add(nd.getNodeValue());
				else 
					classes.add(node.getAttributes().getNamedItem("IDENTIFIER").getNodeValue());
			}
		}
		return classes;
	}
	public static Document loadReqIFModel(IFile f){
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = null;
		try {
			builder = dbf.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
	    try {
			return builder.parse(f.getContents());
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (CoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}
	public static String convertCatalogToText(TreeDB db){
		Document d = convertCatalogToDocument(db);
		StringWriter writer = new StringWriter();
		StreamResult result = new StreamResult(writer);
		TransformerFactory tf = TransformerFactory.newInstance();
		try {
			Transformer transformer = tf.newTransformer();
			transformer.transform(new DOMSource(d), result);
		} catch (TransformerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return writer.toString();
	}
	public static Document convertCatalogToDocument(TreeDB db){
		Map<AttributeType,String> attrTypeToRmf = new HashMap<AttributeType, String>(); 
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = null;
		try {
			builder = dbf.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		Map<String,Map<String,String>> amap = new HashMap<String, Map<String,String>>();
		Document document = (Document) builder.newDocument();
		document.setXmlStandalone(true);
		Element root = document.createElement("REQ-IF");
		root.setAttribute("xmlns", "http://www.omg.org/spec/ReqIF/20110401/reqif.xsd");
		root.setAttribute("xmlns:configuration", "http://eclipse.org/rmf/pror/toolextensions/1.0");
		document.appendChild(root);
		Element header = document.createElement("THE-HEADER");
		root.appendChild(header);
		Element tihe = document.createElement("REQ-IF-HEADER");
		header.appendChild(tihe);
		header = tihe;
		Element corec = document.createElement("CORE-CONTENT");
		root.appendChild(corec);
		Element toole = document.createElement("TOOL-EXTENSIONS");
		root.appendChild(toole);
		//Header formed
		Element temp = document.createElement("COMMENT");
		temp.setTextContent("Exported from Requality");
		header.appendChild(temp);
		temp = document.createElement("CREATION-TIME");
		SimpleDateFormat fm1 = new SimpleDateFormat("yyyy-MM-dd"),
						 fm2 = new SimpleDateFormat("hh:mm:ss");
		temp.setTextContent(fm1.format(new Date())+"T"+fm2.format(new Date()));
		header.appendChild(temp);
		temp = document.createElement("REQ-IF-TOOL-ID");
		temp.setTextContent("Requality (http://requality.ru)");
		header.appendChild(temp);
		temp = document.createElement("REQ-IF-VERSION");
		temp.setTextContent("1.0.1");
		header.appendChild(temp);
		temp = document.createElement("SOURCE-TOOL-ID");
		temp.setTextContent("Requality (http://requality.ru)");
		header.appendChild(temp);
		temp = document.createElement("TITLE");
		String projname = TreesTracker.getTracker().getProject(db).getName();
		temp.setTextContent(projname);
		header.appendChild(temp);
		
		Element reqifc = document.createElement("REQ-IF-CONTENT");
		corec.appendChild(reqifc);
		
		Element ddef = document.createElement("DATATYPES");
		reqifc.appendChild(ddef);
		//Below standard attrubute types
		temp = document.createElement("DATATYPE-DEFINITION-STRING");
		//String
		String uid = "rmf-"+UUID.randomUUID().toString();
		temp.setAttribute("IDENTIFIER", uid);
		temp.setAttribute("LONG-NAME", "String");
		temp.setAttribute("MAX-LENGTH", "32000");
		ddef.appendChild(temp);
		attrTypeToRmf.put(AttributeType.STRING, uid);
		
		temp = document.createElement("DATATYPE-DEFINITION-INTEGER");
		uid = "rmf-"+UUID.randomUUID().toString();
		temp.setAttribute("IDENTIFIER", uid);
		temp.setAttribute("LONG-NAME", "Int");
		ddef.appendChild(temp);
		attrTypeToRmf.put(AttributeType.INT, uid);
		
		temp = document.createElement("DATATYPE-DEFINITION-REAL");
		uid = "rmf-"+UUID.randomUUID().toString();
		temp.setAttribute("IDENTIFIER", uid);
		temp.setAttribute("LONG-NAME", "Float");
		ddef.appendChild(temp);
		attrTypeToRmf.put(AttributeType.FLOAT, uid);
		
		temp = document.createElement("DATATYPE-DEFINITION-BOOLEAN");
		uid = "rmf-"+UUID.randomUUID().toString();
		temp.setAttribute("IDENTIFIER", uid);
		temp.setAttribute("LONG-NAME", "Boolean");
		ddef.appendChild(temp);
		attrTypeToRmf.put(AttributeType.BOOL, uid);
		
		//TODO resource, reference,list
		Element spectypes = document.createElement("SPEC-TYPES");
		reqifc.appendChild(spectypes);
		Map<String,Element> typemap = new HashMap<String, Element>();
		Map<String,String> typeid = new HashMap<String, String>();
		for(String nt:db.getTypes().keySet())
		{
			temp = document.createElement("SPEC-OBJECT-TYPE");
			String id = "rmf-"+UUID.randomUUID().toString();
			typeid.put(nt, id);
			temp.setAttribute("IDENTIFIER", id);
			temp.setAttribute("LONG-NAME", nt);
			spectypes.appendChild(temp);
			
			Element t2 = document.createElement("SPEC-ATTRIBUTES");
			temp.appendChild(t2);
			typemap.put(nt, t2);
		}
		String specuid="rmf-"+UUID.randomUUID().toString();
		Element spect = document.createElement("SPECIFICATION-TYPE");
		spectypes.appendChild(spect);
		spect.setAttribute("IDENTIFIER", specuid);
		spect.setAttribute("LONG-NAME", "Requality Catalog");
		
		Element specobjects = document.createElement("SPEC-OBJECTS");
		reqifc.appendChild(specobjects);
		
		Element specrelations = document.createElement("SPEC-RELATIONS");
		reqifc.appendChild(specrelations);
		
		Element specification = document.createElement("SPECIFICATIONS");
		reqifc.appendChild(specification);
		uid = projname;
		Element specRoot = document.createElement("SPECIFICATION");
		specification.appendChild(specRoot);
		specRoot.setAttribute("IDENTIFIER", projname);
		specRoot.setAttribute("LONG-NAME", projname);
		Element specType = document.createElement("TYPE");
		specRoot.appendChild(specType);
		Element spp = document.createElement("SPECIFICATION-TYPE-REF");
		specType.appendChild(spp);
		spp.setTextContent(specuid);
		Element chi = document.createElement("CHILDREN");
		specRoot.appendChild(chi);
		
		DeepFirstTreeWalker dftw = new DeepFirstTreeWalker(db.getNode(Requirement.getTypeRootQId()));
		Iterator<TreeNode> it = dftw.iterator();
		Map<TreeNode, Element>els= new HashMap<TreeNode, Element>();
		Map<TreeNode, Element>elsP= new HashMap<TreeNode, Element>();
		
		while(it.hasNext())
		{
			TreeNode node = it.next();
			
			String reqid = node.getQualifiedId(); 
			Element reqo = document.createElement("SPEC-OBJECT");
			specobjects.appendChild(reqo);
			reqo.setAttribute("IDENTIFIER", reqid);
			reqo.setAttribute("LONG-NAME", node.getId());
			Element attrs = document.createElement("VALUES");
			reqo.appendChild(attrs);
			String tp = node.getType();
			List<Attribute> lattrs = new ArrayList<Attribute>(node.getAttributes().values());
			lattrs.add(new Attribute(node,AttributeType.STRING,"_uuid",node.getUUId().toString()));
			lattrs.add(new Attribute(node,AttributeType.STRING,"_id",node.getId()));
			for(Attribute a:lattrs){
				String tagname = "STRING";
				switch(a.getType()){
					case FLOAT:{
						tagname= "REAL";
					}break;
					case INT:{
						tagname= "INTEGER";
					}break;
					case BOOL:{
						tagname= "BOOLEAN";
					}break;
					case LIST:{
						//TODO support lists
						continue;
					}
				}
				if(amap.containsKey(node.getType())&&amap.get(node.getType()).containsKey(a.getKey()))
					uid = amap.get(node.getType()).get(a.getKey());
				else{
					if(!amap.containsKey(node.getType()))
						amap.put(node.getType(), new HashMap<String, String>());
					uid ="rmf-"+UUID.randomUUID().toString(); 
					amap.get(node.getType()).put(a.getKey(), uid);
					Element atype = document.createElement("ATTRIBUTE-DEFINITION-"+tagname);
					atype.setAttribute("IDENTIFIER", uid);
					atype.setAttribute("LONG-NAME", a.getKey());
					Element tmp = document.createElement("TYPE");
					Element tmp2 = document.createElement("DATATYPE-DEFINITION-"+tagname+"-REF");
					tmp2.setTextContent(attrTypeToRmf.get(a.getType()));
					tmp.appendChild(tmp2);
					atype.appendChild(tmp);
					typemap.get(node.getType()).appendChild(atype);
				}
				Element aval=document.createElement("ATTRIBUTE-VALUE-"+tagname);
				aval.setAttribute("THE-VALUE", a.getValue().toString().replace("\"", "\\\""));
				attrs.appendChild(aval);
				Element t = document.createElement("DEFINITION");
				aval.appendChild(t);
				Element t2 = document.createElement("ATTRIBUTE-DEFINITION-"+tagname+"-REF");
				t.appendChild(t2);
				t2.setTextContent(uid);
			}
			Element rtype = document.createElement("TYPE");
			reqo.appendChild(rtype);
			Element rref = document.createElement("SPEC-OBJECT-TYPE-REF");
			rtype.appendChild(rref);
			rref.setTextContent(typeid.get(node.getType()));
			if(!els.containsKey(node)){
				String id = "rmf-"+UUID.randomUUID().toString();
				Element specH = document.createElement("SPEC-HIERARCHY");
				specH.setAttribute("IDENTIFIER", id);
				TreeNode par = node.getParent();
				if(els.containsKey(par)){
					if(!elsP.containsKey(par))
						{
							Element ch = document.createElement("CHILDREN");
							els.get(par).appendChild(ch);
							elsP.put(par,ch);
						}
					elsP.get(par).appendChild(specH);
				}else
					chi.appendChild(specH);
				els.put(node, specH);
				
				Element obj = document.createElement("OBJECT");
				specH.appendChild(obj);
				Element obji = document.createElement("SPEC-OBJECT-REF");
				obj.appendChild(obji);
				obji.setTextContent(reqid);
			}
		}
		return document;
	}
	public static Map<String, Set<String>> loadReqIFAttributes(Document model,
			Set<String> types) {
		Set<String> classes = new HashSet<String>();
		Map<String,Set<String>> attrsMap = new HashMap<String, Set<String>>();
		Node n = model.getChildNodes().item(0);
		n = model.getFirstChild();
		if(n==null||!n.getNodeName().equals("REQ-IF"))
			return null;
		n = getFirstChild(n,"CORE-CONTENT");
		n = getFirstChild(n,"REQ-IF-CONTENT");
		n = getFirstChild(n,"SPEC-TYPES");
		for(int j = 0;j<n.getChildNodes().getLength();j++){
			Node node = n.getChildNodes().item(j);
			if(node.getNodeName().equals("SPEC-OBJECT-TYPE")){
				Node nd = node.getAttributes().getNamedItem("LONG-NAME");
				String name = null;
				if(nd!=null)
					name = nd.getNodeValue();
				else 
					name = node.getAttributes().getNamedItem("IDENTIFIER").getNodeValue();
				if(types.contains(name)){
					attrsMap.put(name, new HashSet<String>());
					attrsMap.get(name).add("Identifier");
					node = getFirstChild(node,"SPEC-ATTRIBUTES");
					if(node!=null)
						for(int k = 0;k<node.getChildNodes().getLength();k++){
							Node nde = node.getChildNodes().item(k);
							if(nde.getNodeName().startsWith("ATTRIBUTE-DEFINITION-"))
							{
								String aname = null;
								Node nd2 = nde.getAttributes().getNamedItem("LONG-NAME");
								if(nd2!=null)
									aname = nd2.getNodeValue();
								else 
									aname = nde.getAttributes().getNamedItem("IDENTIFIER").getNodeValue();
								attrsMap.get(name).add(aname);
							}
						}
					
				}
			}
		}
		return attrsMap;
	}
	public static void convertToCatalog(Document model, Map<String, String> mapp1,
			Map<String, String> mapp2) {
		Map<String,TreeNode> reqidToRNode = new HashMap<String, TreeNode>();
		Node n = getFirstChild(model, "REQ-IF");
		if(n==null)
			return;
		n = getFirstChild(n, "CORE-CONTENT");
		n = getFirstChild(n, "REQ-IF-CONTENT");
		Node m = n;
		Map<String,String> typename = new HashMap<String, String>();
		Map<String,String> nata = new HashMap<String, String>();
		Map<String,String> atype = new HashMap<String, String>();
		Map<String,String> specnames = new HashMap<String, String>();
		nata.put("Identifier","Identifier");
		n = getFirstChild(m, "SPEC-TYPES");
		for(int j = 0;j<n.getChildNodes().getLength();j++){
			Node node = n.getChildNodes().item(j);
			if(node.getNodeName().equals("SPEC-OBJECT-TYPE")){
				Node nodeName = node.getAttributes().getNamedItem("LONG-NAME"),
					 nodeId = node.getAttributes().getNamedItem("IDENTIFIER");
				String name = null;
				String id = null;
				if(nodeId!=null)
					id = nodeId.getNodeValue();
				if(nodeName!=null)
					name = nodeName.getNodeValue();
				else
					name = id;
				
				typename.put(id,name);
				if(mapp1.containsKey(name)){
					node = getFirstChild(node, "SPEC-ATTRIBUTES");
					if(node!=null)
						for(int k = 0;k<node.getChildNodes().getLength();k++){
							Node nde = node.getChildNodes().item(k);
							if(nde.getNodeName().startsWith("ATTRIBUTE-DEFINITION-"))
							{
								Node opt = nde.getAttributes().getNamedItem("LONG-NAME");
								String aname = null;
								if(opt!=null)
									aname = nde.getAttributes().getNamedItem("LONG-NAME").getNodeValue();
								String aid = nde.getAttributes().getNamedItem("IDENTIFIER").getNodeValue();
								//if(mapp2.containsKey(name+"."+aname))
								nata.put(aid, aname);
								atype.put(aid, nde.getNodeName().substring(22));
							}
						}
					
				}
			}
		}
		
		n = getFirstChild(m, "SPEC-OBJECTS");
		Map<String,String> nodetypetoobj = new HashMap<String, String>();
		Map<String,Map<String,String>> nodeattrvals = new HashMap<String, Map<String,String>>();
		for(int j = 0;j<n.getChildNodes().getLength();j++){
			Node node = n.getChildNodes().item(j);
			if(node.getNodeName().equals("SPEC-OBJECT")){
				Node ndeName = node.getAttributes().getNamedItem("LONG-NAME"),
						 ndeId = node.getAttributes().getNamedItem("IDENTIFIER");
					String name = null;
					if(ndeName!=null)
						name = ndeName.getNodeValue();
					String nodeId = null;
					if(ndeId!=null)
						nodeId = ndeId.getNodeValue();
				Node tp = getFirstChild(node, "TYPE");
				if(tp!=null){
					Node nd = tp.getFirstChild();
					while(nd!=null){
						if(nd.getNodeName().startsWith("SPEC-OBJECT-TYPE-REF"))
							nodetypetoobj.put(nodeId, nd.getTextContent());
						nd = nd.getNextSibling();
					}
				}
				if(!nodeattrvals.containsKey(nodeId))
					nodeattrvals.put(nodeId, new HashMap<String, String>());
				nodeattrvals.get(nodeId).put("Identifier", nodeId);
				Node vl = getFirstChild(node, "VALUES");
				if(vl!=null){
					for(int l = 0;l<vl.getChildNodes().getLength();l++){
						Node nodea = vl.getChildNodes().item(l);
						if(nodea.getNodeName().startsWith("ATTRIBUTE-VALUE-"))
						{
							
							String ref = null;
							Node def = getFirstChild(nodea, "DEFINITION");
							if(def!=null){
								def = def.getFirstChild();
								while(def!=null){
									if(def.getNodeName().startsWith("ATTRIBUTE-DEFINITION-")&&def.getNodeName().endsWith("-REF"))
										ref = def.getTextContent();
									def = def.getNextSibling();
								}
							}
							Node res = nodea.getAttributes().getNamedItem("THE-VALUE");
							if(res!=null)
								nodeattrvals.get(nodeId).put(ref, nodea.getAttributes().getNamedItem("THE-VALUE").getNodeValue());
							else{
								System.out.println("null value");
								nodeattrvals.get(nodeId).put(ref,"");
							}
						}
					}
				}
			}
		}
		n = getFirstChild(m, "SPECIFICATIONS");
		for(int j = 0;j<n.getChildNodes().getLength();j++){
			Node node = n.getChildNodes().item(j);
			if(node.getNodeName().equals("SPECIFICATION")){
				Node nodeName = node.getAttributes().getNamedItem("LONG-NAME"),
						 nodeId = node.getAttributes().getNamedItem("IDENTIFIER");
					String name = null;
					if(nodeName!=null)
						name = nodeName.getNodeValue();
					String id = null;
					if(nodeId!=null)
						id = nodeId.getNodeValue();
				//node = node.getFirstChild();
				Node nde = getFirstChild(node, "CHILDREN");
				List<Node> querry = new ArrayList<Node>();
				for(int k=0;k<nde.getChildNodes().getLength();k++){
					 node = nde.getChildNodes().item(k);
//					 
					 if(node.getNodeType()!=Node.TEXT_NODE)
						 querry.add(node);
				}
				//create project
				IWorkspace workspace = ResourcesPlugin.getWorkspace();
				if(name == null)
					name = id;
				String projName = name;
				IProject project = workspace.getRoot().getProject(projName);
				if(project.exists()){
					int k = 0;
					while(workspace.getRoot().getProject(projName+"("+k+")").exists())
						k++;
					projName = projName+"("+k+")";
					project = workspace.getRoot().getProject(projName);
				}
	            TreeDB db = NewRequalityProjectWizard.createProject(projName);
	            db.startTransaction("import project "+projName);
	            db.getNode(Requirement.getTypeRootQId());
				int i = 0;
				boolean maptoRoot = querry.size() == 1;
				Map<Node, NodeDesc> pars = new HashMap<Node, NodeDesc>();
				NodeDesc root = NodeDesc.fromNode(db.getNode(Requirement.getTypeRootQId()));
				
				while(querry.size()>i){
					Node nd = querry.get(i);
					NodeDesc rnode = null;
					NodeDesc rpar = pars.get(nd);
					Node chi = null;
					String ref = null;
					nd = nd.getFirstChild();
					while(nd!=null){
						if(nd.getNodeName().equals("OBJECT"))
							ref = getFirstChild(nd, "SPEC-OBJECT-REF").getTextContent();
						else if(nd.getNodeName().equals("CHILDREN")){
							chi = nd;
						}
						nd = nd.getNextSibling();
					}
					String fid = null;
					
					String typeref = nodetypetoobj.get(ref);
					String tname = typename.get(typeref);
					String reqname = mapp1.get(tname);
					if(rpar == null)
						rpar = root;
					if(maptoRoot && i == 0)
					{
						rnode = root;
					}else{
						//TODO handle not unique ids in a proper way
						UUID uid = UUID.randomUUID();
						HashMap<String,Attribute> attrs = new HashMap<String, Attribute>();
						attrs.put(TreeNode.ATTR_TYPE, new Attribute(uid,AttributeType.STRING,TreeNode.ATTR_TYPE,reqname!=null?reqname:Requirement.TYPE_NAME));
						rnode = new NodeDesc(uid,
								rpar.getUUId(),RequalityCLI.getNextId(rpar, reqname!=null?reqname:Requirement.TYPE_NAME, ""), 
								attrs, new ArrayList<UUID>());
					}
					if(ref!=null && nodeattrvals.containsKey(ref))
					for(String aref:nodeattrvals.get(ref).keySet()){
						String aname=nata.get(aref),aval = nodeattrvals.get(ref).get(aref);
						//select name
						if(aname==null)
							aname=aref;
						if(mapp2.containsKey(tname+'.'+aname)||mapp2.isEmpty()&&aname.equals("_id")){
							if(mapp2.isEmpty()||mapp2.get(tname+'.'+aname).equals("_id")){
								fid = nodeattrvals.get(ref).get(aref);
								UUID loc = null;
								for(UUID uid:rpar.getChildrenUUId())if(rpar.getChildDesc(uid).getId().equals(fid)){loc=uid;break;}
								if(fid!=null && loc!=null)
								{
									int l = 0;
									{
										loc=null;
										for(UUID uid:rpar.getChildrenUUId())if(rpar.getChildDesc(uid).getId().equals(fid+"("+l+")")){
											loc=uid;
											break;
										}
										l++;
									}
									while(loc!=null);
										
									fid = fid+"("+l+")";
								}
								if(fid!=null)
									rnode.setId(fid);
								//do not create
								aname = null;
							}else
								aname = mapp2.get(tname+'.'+aname);
						}else{
							if(aname.equals("_uuid")){
								UUID newuid = null;
								if(nodeattrvals.get(ref).get(aref)!=null)
									newuid = UUID.fromString(nodeattrvals.get(ref).get(aref));
								if(newuid!=null){
									Map<String,Attribute>newmap = new HashMap<String, Attribute>();
									for(String a:rnode.getAttributes().keySet())
										newmap.put(a, Attribute.deepCopy(rnode.getAttributes().get(a),newuid));
									UUID par = rnode.getParentUUId();
									if(root==rnode)
										par = db.getRootNode().getUUId();
									rnode = new NodeDesc(newuid,par,rnode.getId(), newmap, rnode.getChildrenUUId());
									if(par.equals(db.getRootNode().getUUId()))
										root = rnode;
								}
								//also do not create
								aname = null;
							}
						}
						if(aname!=null&&aval!=null&&!aval.toString().isEmpty()){
							AttributeType at = AttributeType.STRING;
							String rfatype = atype.get(aref);
							if(rfatype!=null){
								if(rfatype.equals("REAL"))
									at = AttributeType.FLOAT;
								else if(rfatype.equals("INTEGER"))
									at = AttributeType.INT;
								else if (rfatype.equals("BOOLEAN"))
									at = AttributeType.BOOL;
								rnode.setAttribute(new Attribute(rnode.getUUId(),at,aname,aval));	
							}
						}
						//create attr
					}
					
					if(chi!=null)
						for(int k =0;k<chi.getChildNodes().getLength();k++){
							Node nd2 = chi.getChildNodes().item(k);
							if(nd2.getNodeType()!=Node.TEXT_NODE){
								querry.add(nd2);
								pars.put(nd2, rnode);
							}
						}
					i++;
					if(rpar!=rnode){
						List<UUID>childs = new ArrayList<UUID>(rpar.getChildrenUUId());
						childs.add(rnode.getUUId());
						rpar.setChildrenUUIDs(childs);
						rpar.setChildrenDesc(rnode.getUUId(), rnode);
					}
				}
				db.removeNode(db.getNode(Requirement.getTypeRootQId()));
				root.createNode(db, true);
				querry.clear();
				pars.clear();
				db.commit();
			}
		}
	}
	private static Node getFirstChild(Node node, String type){
		NodeList nl = node.getChildNodes();
		for(int i=0;i<nl.getLength();i++)
			if(nl.item(i).getNodeName().equals(type))
				return nl.item(i);
		return null;
	}
}
